/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

// BriefcaseSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "BriefcaseSettings.h"
#include "ExploreFolders.h"

#include "comutil.h"

#include "pim/ClientSettings.h"
#include "pim/Maincpp.h"	// for checkStartSync
#include "BriefcaseSettingsAdvanced.h"

// CBriefcaseSettings dialog

IMPLEMENT_DYNAMIC(CBriefcaseSettings, CDialog)

CBriefcaseSettings::CBriefcaseSettings(CWnd* pParent /*=NULL*/)
: CDialog(CBriefcaseSettings::IDD, pParent)
{

}

CBriefcaseSettings::~CBriefcaseSettings()
{
}

void CBriefcaseSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
#if defined(WIN32_PLATFORM_WFSP)
    DDX_Control(pDX, IDC_BRIEFCASE_SPIN1, spinSyncType );
#endif
    DDX_Control(pDX, IDC_BRIEFCASE_SYNCTYPE, lstSyncType );
    DDX_Control(pDX, IDC_BRIEFCASE_LINK_ADVANCED, linkAdvanced);
    DDX_Control(pDX, IDC_BRIEFCASE_LINK_CHOOSE, linkChoose);
}

BEGIN_MESSAGE_MAP(CBriefcaseSettings, CDialog)
    #if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
        ON_WM_SIZE()
    #endif

  ON_COMMAND(ID_MENU_FOLDER, &CBriefcaseSettings::OnMenuFolder)
  ON_BN_CLICKED(IDC_BRIEFCASE_LINK_ADVANCED, &CBriefcaseSettings::OnBnClickedBriefcaseLinkAdvanced)
  ON_BN_CLICKED(IDC_BRIEFCASE_LINK_CHOOSE, &CBriefcaseSettings::OnBnClickedBriefcaseLinkChoose)
  ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


// CBriefcaseSettings message handlers

BOOL CBriefcaseSettings::OnInitDialog()
{
	CString s1;
	s1.LoadString(IDS_BRIEFCASE_SETTINGS); SetWindowText(s1);
	CDialog::OnInitDialog();


  #ifdef WIN32_PLATFORM_WFSP
    spinSyncType.SetRange(3,0);  spinSyncType.SetBuddy(&lstSyncType);
  #endif

    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL;
    #if defined WIN32_PLATFORM_WFSP
    #if (_WIN32_WCE < 0x500)
        dlgCommandBar.nToolBarId = IDR_MENUBAR_OKCANCEL;
    #endif
    #endif
    dlgCommandBar.hInstRes = ::AfxGetInstanceHandle();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    }

 // read string resources
  s1.LoadString(IDS_SETTCOMM_SYNCTYPE1);	lstSyncType.AddString(s1);
  s1.LoadString(IDS_SETTCOMM_SYNCTYPE2);	lstSyncType.AddString(s1);
  s1.LoadString(IDS_SETTCOMM_SYNCTYPE3);	lstSyncType.AddString(s1);

  s1.LoadString(IDS_SYNCTYPE); SetDlgItemText(IDC_BRIEFCASE_STATIC_SYNCTYPE,s1);
  s1.LoadString(IDS_FOLDER_TO_SYNC); SetDlgItemText(IDC_BRIEFCASE_STATIC_FOLDERTOSYNC,s1);
  s1.LoadString(IDS_ADVANCED_SETTINGS); linkAdvanced.setText(s1);
  s1.LoadString(IDS_CHOOSE); linkChoose.setText(s1);

  // set font
  LOGFONT lf;
  memset(&lf, 0, sizeof(LOGFONT));
  HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
  GetObject(hFont, sizeof lf, &lf);
  lf.lfWeight = FW_BOLD;
  lf.lfHeight +=FONT_HEIGHT_OFFSET;
  VERIFY(fontBold.CreateFontIndirect(&lf));
  GetDlgItem(IDC_BRIEFCASE_STATIC_SYNCTYPE)->SetFont(&fontBold);
  GetDlgItem(IDC_BRIEFCASE_STATIC_FOLDERTOSYNC)->SetFont(&fontBold);

  // ppc header
  #if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(IDS_BRIEFCASE_SETTINGS); SetDlgItemText(IDC_HEADER_STATIC, s1);
    GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
  #endif

 // read from the registry
 CString s3; int pos=0;

 WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("briefcase");
 pos = SynctypeReg2UI(ssconf->getSync());
 lstSyncType.SetCurSel(pos);

 s3 = ssconf->getSyncDir().c_str();
 folder = s3;

 if (s3 == "") {
   LPTSTR dir = new  WCHAR[512];
   SHGetSpecialFolderPath(NULL, dir, CSIDL_PERSONAL, FALSE);
    wstring s(dir);
   delete [] dir;
    wstring brief = s; brief += L"\\briefcase";
    if (existsDirectory((wchar_t*)brief.c_str())) {
        s3 = brief.c_str();
    } else {
        s3 = s.c_str();
    }
 }

 folder = s3;
 SetDlgItemText(IDC_BRIEFCASE_STATIC_FOLDER,folder);
 s3.LoadString(IDS_BRIEFCASE_CHOOSE_FOLDER);

 // fix for wrong title problem, we get & set the title for the main window
 wchar_t title[256];
 AfxGetMainWnd()->GetWindowText(title,256);
 AfxGetMainWnd()->SetWindowText(title);

 lstSyncType.SetFocus();
 return FALSE;  // set focus to listbox
}


void CBriefcaseSettings::OnOK()
{
  USES_CONVERSION;
  BOOL dirtyFlag=FALSE;

  if(checkStartSync()){
      CDialog::OnOK();
  }

  // save values in registry
  WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("briefcase");
  CString s1; int pos=0;
  _bstr_t bst;

  pos = lstSyncType.GetCurSel();
  if(strcmp(SynctypeUI2Reg(pos),ssconf->getSync()) != 0){
    ssconf->setSync(SynctypeUI2Reg(pos));
    dirtyFlag = TRUE;
  }

  if(folder == _T("")) {
      folder = ssconf->getSyncDir().c_str();
  }
  bst = folder.AllocSysString();
  if(strcmp(T2A(folder), ssconf->getSyncDir().c_str()) != 0 ){
    ssconf->setSyncDir(T2A(folder));
    dirtyFlag = TRUE;
  }

  if(dirtyFlag){
    getRegConfig()->setDirty(CS_DIRTY_SOURCE_BRIEFCASE);
    // bst.Detach();
    getRegConfig()->saveDirty();
  }

  CDialog::OnOK();
}

void CBriefcaseSettings::OnMenuFolder()
{
    CExploreFolders wndExplore;

    #if defined(WIN32_PLATFORM_PSPC)
        wndExplore.setIsNotes(false);
    #endif

    wndExplore.setFolder(folder);

    INT_PTR result = wndExplore.DoModal();
    if (result == IDCHOOSE){
        //wstring s = wndExplore.getFolder();
        folder = wndExplore.getFolder();
        //SetDlgItemText(IDC_BRIEFCASE_STATIC_FOLDER, s.c_str());
        SetDlgItemText(IDC_BRIEFCASE_STATIC_FOLDER, folder);
    }

}

#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
void CBriefcaseSettings::OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/)
{
    int resId;
    switch(DRA::GetDisplayMode())
    {
    case DRA::Landscape:
        resId = IDD_UI_BRIEFCASE_WIDE; break;
    case DRA::Portrait:
        resId = IDD_UI_BRIEFCASE; break;
    case DRA::Square:
        resId = IDD_UI_BRIEFCASE_SQUARE; break;
    default:
        resId = IDD_UI_BRIEFCASE; break;
    };

    DRA::RelayoutDialog( AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));

    CString s1;

    s1.LoadString(IDS_SYNCTYPE); SetDlgItemText(IDC_BRIEFCASE_STATIC_SYNCTYPE,s1);
    s1.LoadString(IDS_FOLDER_TO_SYNC); SetDlgItemText(IDC_BRIEFCASE_STATIC_FOLDERTOSYNC,s1);

    #if defined(WIN32_PLATFORM_PSPC)
        s1.LoadString(IDS_BRIEFCASE_SETTINGS); SetDlgItemText(IDC_HEADER_STATIC, s1);
    #endif

    s1=folder;
    SetDlgItemText(IDC_BRIEFCASE_STATIC_FOLDER,s1);
}
#endif


void CBriefcaseSettings::OnBnClickedBriefcaseLinkAdvanced(){
    CBriefcaseSettingsAdvanced wndBriefcaseAdvanced;

    INT_PTR result = wndBriefcaseAdvanced.DoModal();
}


// capture pressing of 'Advanced' link
BOOL CBriefcaseSettings::PreTranslateMessage(MSG* pMsg){
    bool bProcessed =false;
    if(pMsg->message == WM_KEYDOWN){
        if(pMsg->wParam == VK_RETURN){
            if(GetFocus()->GetSafeHwnd() == linkAdvanced.GetSafeHwnd()){
                bProcessed = true;
                OnBnClickedBriefcaseLinkAdvanced();
            }
            else
                if(GetFocus()->GetSafeHwnd() == linkChoose.GetSafeHwnd()){
                bProcessed = true;
                OnMenuFolder();
            }


        }
    };

    if(bProcessed)
        return TRUE;
    else
        return CDialog::PreTranslateMessage(pMsg);
}

void CBriefcaseSettings::OnBnClickedBriefcaseLinkChoose(){
    OnMenuFolder();
}

HBRUSH CBriefcaseSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
#if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
#endif

    return hbr;
}