/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

// ContactSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "ContactSettings.h"
#include "ContactSettingsAdvanced.h"

#include "ClientUtil.h"
#include "examples/config.h"
#include "base/fscapi.h"
#include "spds/constants.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/ManagementNode.h"

#include "pim/ClientSettings.h"
#include "pim/Maincpp.h"

// CContactSettings dialog

IMPLEMENT_DYNAMIC(CContactSettings, CDialog)

CContactSettings::CContactSettings(CWnd* pParent /*=NULL*/)
	: CDialog(CContactSettings::IDD, pParent)
{

}

CContactSettings::~CContactSettings()
{
}

void CContactSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
#if defined(WIN32_PLATFORM_WFSP)
    DDX_Control(pDX, IDC_SETTCONT_SPIN1, spinSyncType );
#endif
    DDX_Control(pDX, IDC_CONT_SYNCTYPE, lstSyncType );
    DDX_Control(pDX, IDC_CONTACTS_LINK_ADVANCED, linkAdvanced);
}


BEGIN_MESSAGE_MAP(CContactSettings, CDialog)
    ON_BN_CLICKED(IDC_CONTACTS_LINK_ADVANCED, &CContactSettings::OnBnClickedContactsLinkAdvanced)
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


// CContactSettings message handlers


BOOL CContactSettings::OnInitDialog(){
// SET window title
CString s1;
s1.LoadString(IDS_CONTACTS_SETTINGS) ;SetWindowText(s1);
CDialog::OnInitDialog();

  #ifdef WIN32_PLATFORM_WFSP
    spinSyncType.SetRange(4,0);	spinSyncType.SetBuddy(&lstSyncType);
  #endif

  ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
  dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
  dlgCommandBar.hwndParent = this->GetSafeHwnd();
  dlgCommandBar.dwFlags = SHCMBF_HMENU;
  dlgCommandBar.nToolBarId = IDM_OKCANCEL;
  #if defined WIN32_PLATFORM_WFSP
    #if (_WIN32_WCE < 0x500)
        dlgCommandBar.nToolBarId = IDR_MENUBAR_OKCANCEL;
    #endif
  #endif
  dlgCommandBar.hInstRes = ::AfxGetInstanceHandle();
  dlgCommandBar.nBmpId = 0;
  dlgCommandBar.cBmpImages = 0;
  if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
  };

 // read string resources
 s1.LoadString(IDS_SETTCOMM_SYNCTYPE1); lstSyncType.AddString(s1);
 s1.LoadString(IDS_SETTCOMM_SYNCTYPE2);	lstSyncType.AddString(s1);
 s1.LoadString(IDS_SETTCOMM_SYNCTYPE3);	lstSyncType.AddString(s1);

 s1.LoadString(IDS_ADVANCED_SETTINGS); linkAdvanced.setText(s1);

 LoadStringResources();

 LOGFONT lf;
 memset(&lf, 0, sizeof(LOGFONT));
 HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
 GetObject(hFont, sizeof lf, &lf);
 lf.lfWeight = FW_BOLD;
 lf.lfHeight +=FONT_HEIGHT_OFFSET;
 VERIFY(fontBold.CreateFontIndirect(&lf));

 GetDlgItem(IDC_SETTCONT_STATIC_SYNCTYPE)->SetFont(&fontBold);

 // set ppc header
 #if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(IDS_CONTACTS_SETTINGS); SetDlgItemText(IDC_HEADER_STATIC, s1);
    GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
 #endif

// read from the registry
 WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("contact");
 CString s3; int pos=0;

 pos = SynctypeReg2UI(ssconf->getSync());
 lstSyncType.SetCurSel(pos);

 // check if another sync is in progress
  if(checkStartSync() != 0){
   lstSyncType.EnableWindow(FALSE);
   #if defined(WIN32_PLATFORM_WFSP)
     spinSyncType.EnableWindow(FALSE);
   #endif
  }
  else{
	}

 // fix for wrong title problem, we get & set the title for the main window
 wchar_t title[256];
 AfxGetMainWnd()->GetWindowText(title,256);
 AfxGetMainWnd()->SetWindowText(title);

 lstSyncType.SetFocus();
 return FALSE;
}

void CContactSettings::OnOK()
{
  if(checkStartSync()){
      CDialog::OnOK();
  }
  // save values in registry
  WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("contact");
  CString s1; int pos=0;

  pos = lstSyncType.GetCurSel();
  if(strcmp(SynctypeUI2Reg(pos),ssconf->getSync()) != 0){
    ssconf->setSync(SynctypeUI2Reg(pos));

    getRegConfig()->setDirty(CS_DIRTY_SOURCE_CONTACTS);
    getRegConfig()->saveDirty();
  }

  CDialog::OnOK();
}

void CContactSettings::LoadStringResources(){
    CString s1;
    s1.LoadString(IDS_SYNCTYPE); SetDlgItemText(IDC_SETTCONT_STATIC_SYNCTYPE, s1);
}

void CContactSettings::OnBnClickedContactsLinkAdvanced()
{
    CContactSettingsAdvanced wndContactsAdvanced;
    INT_PTR result = wndContactsAdvanced.DoModal();
}


// capture pressing of 'Advanced' link
BOOL CContactSettings::PreTranslateMessage(MSG* pMsg){
    bool bProcessed =false;
    if(pMsg->message == WM_KEYDOWN){
        if(pMsg->wParam == VK_RETURN){
            if(GetFocus()->GetSafeHwnd() == linkAdvanced.GetSafeHwnd()){
                bProcessed = true;
                OnBnClickedContactsLinkAdvanced();
            }

        }
    };

    if(bProcessed)
        return TRUE;
    else
        return CDialog::PreTranslateMessage(pMsg);
}

HBRUSH CContactSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
    #if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
    #endif

    return hbr;
}