/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */
// ContactsSettingsAdvanced.cpp : implementation file
//

#include "stdafx.h"
#include "ContactSettingsAdvanced.h"
#include "pim/maincpp.h"
#include "pim/clientsettings.h"
#include "comutil.h"
#include "tpcshell.h"

// CContactSettingsAdvanced dialog

IMPLEMENT_DYNAMIC(CContactSettingsAdvanced, CDialog)

CContactSettingsAdvanced::CContactSettingsAdvanced(CWnd* pParent /*=NULL*/)
	: CDialog(CContactSettingsAdvanced::IDD, pParent)
{

}

CContactSettingsAdvanced::~CContactSettingsAdvanced(){
  #if defined(WIN32_PLATFORM_PSPC)
    editRemote.Detach();
  #endif
}

void CContactSettingsAdvanced::DoDataExchange(CDataExchange* pDX)
{
    DDX_Control(pDX, IDC_SETTCONT_RADIO1, radioBut1);
    DDX_Control(pDX, IDC_SETTCONT_RADIO2, radioBut2);
    DDX_Control(pDX, IDC_SETTCONT_EDIT_REMOTE, editRemote);
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CContactSettingsAdvanced, CDialog)
#ifdef WIN32_PLATFORM_WFSP
    ON_MESSAGE(WM_HOTKEY,&CContactSettingsAdvanced::OnHotkey)
#endif
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()

BOOL CContactSettingsAdvanced::OnInitDialog(){
    CString s1;

    s1.LoadString(IDS_ADVANCED_CONTACTS); SetWindowText(s1);
    CDialog::OnInitDialog();

    // create menu bar
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL;
    #if defined WIN32_PLATFORM_WFSP
    #if (_WIN32_WCE < 0x500)
        dlgCommandBar.nToolBarId = IDR_MENUBAR_OKCANCEL;
    #endif
    #endif
    dlgCommandBar.hInstRes = ::AfxGetInstanceHandle();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    };

    editRemote.SetLimitText(EDIT_MAXLENGTH);
    LoadStringResources();

    // set font
    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    GetDlgItem(IDC_SETTCONT_STATIC_REMONAME)->SetFont(&fontBold);
    GetDlgItem(IDC_SETTCONT_STATIC_SYNCFORM)->SetFont(&fontBold);
    // set ppc header
    #if defined(WIN32_PLATFORM_PSPC)
        GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
    #endif

    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("contact");

    s1 = ssconf->getURI();
    SetDlgItemText(IDC_SETTCONT_EDIT_REMOTE,s1);
    if(ssconf->getUseSif() == "1"){
        radioBut1.SetCheck(BST_CHECKED);
        radioBut2.SetCheck(BST_UNCHECKED);
    }
    else{
        radioBut2.SetCheck(BST_CHECKED);
        radioBut1.SetCheck(BST_UNCHECKED);
    };

    // set a large int as current selection, so the cursor will be at end
    editRemote.SetSel(LARGE_INT ,LARGE_INT);
    editRemote.SetFocus();

#if defined(WIN32_PLATFORM_WFSP)
    // override back key
    LPARAM lparam = MAKELPARAM(SHMBOF_NODEFAULT | SHMBOF_NOTIFY,
        SHMBOF_NODEFAULT | SHMBOF_NOTIFY);
    ::SendMessage(dlgCommandBar.hwndMB, SHCMBM_OVERRIDEKEY, VK_TBACK, lparam);
#endif

    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    return FALSE;
}


void CContactSettingsAdvanced::LoadStringResources(){
    CString s1;

    s1.LoadString(IDS_SETTCONT_FORMAT1); SetDlgItemText(IDC_SETTCONT_RADIO1, s1);
    s1.LoadString(IDS_SETTCONT_FORMAT2);SetDlgItemText(IDC_SETTCONT_RADIO2, s1);
    s1.LoadString(IDS_REMOTENAME); SetDlgItemText(IDC_SETTCONT_STATIC_REMONAME,s1);
    s1.LoadString(IDS_CONTACT_SYNCFORMAT); SetDlgItemText(IDC_SETTCONT_STATIC_SYNCFORM,s1);

    // set ppc header
    #if defined(WIN32_PLATFORM_PSPC)
        s1.LoadString(IDS_ADVANCED_CONTACTS); SetDlgItemText(IDC_HEADER_STATIC, s1);
    #endif
}

afx_msg void CContactSettingsAdvanced::OnOK(){

    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("contact");
    CString s1;
    _bstr_t bst;
    BOOL dirtyFlag=FALSE;

    if(checkStartSync()){
        CDialog::OnOK();
    };

    GetDlgItemText(IDC_SETTCONT_EDIT_REMOTE, s1);
    bst.Assign(s1.AllocSysString());
    if(strcmp((char*)bst,ssconf->getURI()) != 0){
        ssconf->setURI((char*)bst);
        dirtyFlag=TRUE;
    }

    bst.Detach();

    if (radioBut1.GetCheck() == BST_CHECKED){
        if(strcmp(ssconf->getUseSif().c_str(),"1") != 0){
            ssconf->setUseSif("1");
            dirtyFlag=TRUE;
        }
    }
    else{
        if(strcmp(ssconf->getUseSif().c_str(),"0") != 0){
            ssconf->setUseSif("0");
            dirtyFlag=TRUE;
        }
    }

    if(dirtyFlag){
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_CONTACTS);
        getRegConfig()->saveDirty();
    }

    CDialog::OnOK();
}

#ifdef WIN32_PLATFORM_WFSP
LRESULT CContactSettingsAdvanced::OnHotkey(WPARAM wParam, LPARAM lParam)
{
    if (HIWORD(lParam) == VK_TBACK){
        SHSendBackToFocusWindow(WM_HOTKEY, wParam, lParam);
    }
    return 0;
}
#endif

BOOL CContactSettingsAdvanced::PreTranslateMessage( MSG* pMsg ){
    // change radio buttons selection
    // this MUST be changed if the layout of the dialog changes

    bool bProcessed =false;
    HWND wndFocus = GetFocus()->GetSafeHwnd();

    if(pMsg->message == WM_KEYDOWN){
        if( (pMsg->wParam == VK_LEFT) && (wndFocus == radioBut1.GetSafeHwnd()) ){
            radioBut2.SetFocus();
            radioBut2.SetCheck(BST_CHECKED);
            radioBut1.SetCheck(BST_UNCHECKED);
            bProcessed = true;
        };

        if( (pMsg->wParam == VK_RIGHT) && (wndFocus == radioBut2.GetSafeHwnd()) ){
            radioBut1.SetFocus();
            radioBut1.SetCheck(BST_CHECKED);
            radioBut2.SetCheck(BST_UNCHECKED);
            bProcessed = true;
        };
    }

    if(bProcessed){
        return TRUE;
    }
    else{
        return CDialog::PreTranslateMessage(pMsg);
    }
}


HBRUSH CContactSettingsAdvanced::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
    #if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
    #endif

    return hbr;
}