/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

// DeviceInfo.cpp : implementation file
//

#include "stdafx.h"
#include "DeviceInfo.h"
#include "ui.h"
#include "pim/SettingFunctions.h"
#include "pim/ClientSettings.h"


// CDeviceInfo dialog

IMPLEMENT_DYNAMIC(CDeviceInfo, CDialog)

CDeviceInfo::CDeviceInfo(CWnd* pParent /*=NULL*/)
	: CDialog(CDeviceInfo::IDD, pParent)
{

}

CDeviceInfo::~CDeviceInfo()
{
}

void CDeviceInfo::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BOOL CDeviceInfo::OnInitDialog(){
    CString s1;
    CString devId;

    s1.LoadString(IDS_DEVINFO_TITLE);
    SetWindowText(s1);
    CDialog::OnInitDialog();

    // set menubar
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_SK1_CLOSE;
    #if defined WIN32_PLATFORM_WFSP
        #if (_WIN32_WCE < 0x500)
            dlgCommandBar.nToolBarId = IDR_MENUBAR_CLOSE;
        #endif
    #endif
    dlgCommandBar.hInstRes = ::AfxGetInstanceHandle();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    }

    // create font
    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    // load string resources
    s1.LoadString(IDS_SETTCOMM_DEVID); SetDlgItemText(IDC_DEVINFO_STATIC_DEVID, s1);
    s1.LoadString(IDS_SETTCOMM_CURRIP); SetDlgItemText(IDC_DEVINFO_STATIC_IP, s1);

    // set font
    GetDlgItem(IDC_DEVINFO_STATIC_DEVID)->SetFont(&fontBold);
    GetDlgItem(IDC_DEVINFO_STATIC_IP)->SetFont(&fontBold);

    // set ppc header
    #if defined(WIN32_PLATFORM_PSPC)
        s1.LoadString(IDS_DEVINFO_TITLE); SetDlgItemText(IDC_HEADER_STATIC, s1);
        GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
    #endif

    // load settings from Registry
    ClientSettings* cs = getRegConfig();
    devId = cs->getDeviceConfig().getDevID();

    if (devId == "") {
        #if defined(WIN32_PLATFORM_WFSP)
            devId = getIMEI().c_str();
        #endif
        #if defined(WIN32_PLATFORM_PSPC)
            devId = GetSerialNumberFromKernelIoControl().c_str();
        #endif
    }

    SetDlgItemText(IDC_DEVINFO_DEVID, devId);

    s1 = cs->getIp().c_str();
    SetDlgItemText(IDC_DEVINFO_IP, s1);

    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    return FALSE;
}

BEGIN_MESSAGE_MAP(CDeviceInfo, CDialog)
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


// CDeviceInfo message handlers

HBRUSH CDeviceInfo::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
    #if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
    #endif

    return hbr;
}