/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

// LoglevelSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "LoglevelSettings.h"

#include "ClientUtil.h"
#include "examples/config.h"
#include "base/fscapi.h"
#include "base/Log.h"
#include "spds/constants.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/ManagementNode.h"

#include "pim/ClientSettings.h"
#include "pim/Maincpp.h"

#include <string>

// CLoglevelSettings dialog

IMPLEMENT_DYNAMIC(CLoglevelSettings, CDialog)

CLoglevelSettings::CLoglevelSettings(CWnd* pParent /*=NULL*/)
: CDialog(CLoglevelSettings::IDD, pParent)
{
    logname = "\\";
    logname += LOG_NAME;
}

CLoglevelSettings::~CLoglevelSettings()
{
}

void CLoglevelSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_SETTLOG_RADIO1, radioBut1);
    DDX_Control(pDX, IDC_SETTLOG_RADIO2, radioBut2);
    DDX_Control(pDX, IDC_SETTLOG_RADIO3, radioBut3);
}


BEGIN_MESSAGE_MAP(CLoglevelSettings, CDialog)
    ON_COMMAND(IDVIEW, &CLoglevelSettings::OnMenuViewLog)
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


// CLoglevelSettings message handlers

BOOL CLoglevelSettings::OnInitDialog()
{
    CString s1;
    s1.LoadString(IDS_LOGGING); SetWindowText(s1);
    CDialog::OnInitDialog();

    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_SAVEVIEW;
    #if defined WIN32_PLATFORM_WFSP
        #if (_WIN32_WCE < 0x500)
            dlgCommandBar.nToolBarId = IDR_MENUBAR_SAVEVIEW;
        #endif
    #endif
    dlgCommandBar.hInstRes = ::AfxGetInstanceHandle();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    }

    // load string resources
    s1.LoadString(IDS_SETTLOG_RADIO1); SetDlgItemText(IDC_SETTLOG_RADIO1, s1);
    s1.LoadString(IDS_SETTLOG_RADIO2); SetDlgItemText(IDC_SETTLOG_RADIO2, s1);
    s1.LoadString(IDS_SETTLOG_RADIO3); SetDlgItemText(IDC_SETTLOG_RADIO3, s1);
    s1.LoadString(IDS_LOGLEVEL_SETTINGS); SetDlgItemText(IDC_SETTLOG_STATIC, s1);
    s1.LoadString(IDS_LOG_SELECT_LATEST); SetDlgItemText(IDC_LOG_STATIC_LATEST, s1);

    // create & set font
    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    GetDlgItem(IDC_SETTLOG_STATIC)->SetFont(&fontBold);
    GetDlgItem(IDC_LOG_STATIC_LATEST)->SetFont(&fontBold);

    // set ppc header
    #if defined(WIN32_PLATFORM_PSPC)
        s1.LoadString(IDS_LOGGING); SetDlgItemText(IDC_HEADER_STATIC, s1);
        GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
    #endif

    // read from the registry
    ClientSettings* cs = getRegConfig();
    int value;
    value = cs->getDeviceConfig().getLogLevel();
    switch(value) {
        case 0:
            radioBut1.SetCheck(BST_CHECKED);
            radioBut1.SetFocus();
            break;
        case 1:
            radioBut2.SetCheck(BST_CHECKED);
            radioBut2.SetFocus();
            break;
        case 2:
            radioBut3.SetCheck(BST_CHECKED);
            radioBut3.SetFocus();
            break;
    }

    // check if the log file exists
    FILE* f = _wfopen(logname, TEXT("r"));
    if(f == NULL){
        // hide text and disable 'View' button
        HWND hwndMB = SHFindMenuBar (this->GetSafeHwnd());

        TBBUTTONINFO tbi;
        memset (&tbi, 0, sizeof (tbi));
        tbi.cbSize = sizeof (tbi);
        tbi.dwMask = TBIF_STATE;
        ::SendMessage (hwndMB, TB_GETBUTTONINFO, IDVIEW, (LPARAM)&tbi);
        tbi.fsState &= ~TBSTATE_ENABLED;
        ::SendMessage (hwndMB, TB_SETBUTTONINFO, IDVIEW, (LPARAM)&tbi);

        GetDlgItem(IDC_LOG_STATIC_LATEST)->ShowWindow(FALSE);
    }
    else {
        fclose(f);
    }

    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    return FALSE;
}

void CLoglevelSettings::OnOK()
{
    if(checkStartSync()){
        CDialog::OnOK();
    }
    BOOL dirtyFlag=FALSE;

    // save values in registry
    ClientSettings* cs = getRegConfig();

    if (radioBut1.GetCheck() == BST_CHECKED){
        if(cs->getDeviceConfig().getLogLevel() != 0){
            cs->getDeviceConfig().setLogLevel(LOG_LEVEL_NONE);   // None
            dirtyFlag=TRUE;
        }
    }
    if (radioBut2.GetCheck() == BST_CHECKED){
        if(cs->getDeviceConfig().getLogLevel() != 1){
            cs->getDeviceConfig().setLogLevel(LOG_LEVEL_INFO);   // Info
            dirtyFlag=TRUE;
        }
    }

    if (radioBut3.GetCheck() == BST_CHECKED){
        if(cs->getDeviceConfig().getLogLevel()!= 2){
            cs->getDeviceConfig().setLogLevel(LOG_LEVEL_DEBUG);   // Debug
            dirtyFlag=TRUE;
        }
    }

    if(dirtyFlag){
        getRegConfig()->setDirty(CS_DIRTY_DEVICE_CONFIG);
        getRegConfig()->saveDirty();
    }

    CDialog::OnOK();
}


void CLoglevelSettings::OnMenuViewLog()
{
    SHELLEXECUTEINFO lpExecInfo;
    memset(&lpExecInfo, 0, sizeof(SHELLEXECUTEINFO));
    lpExecInfo.cbSize = sizeof(SHELLEXECUTEINFO);

    lpExecInfo.lpFile = logname;
    lpExecInfo.nShow = SW_SHOWNORMAL;
    lpExecInfo.fMask = SEE_MASK_NOCLOSEPROCESS;
    lpExecInfo.lpVerb = _T("open");
    ShellExecuteEx(&lpExecInfo);
}

BOOL CLoglevelSettings::PreTranslateMessage( MSG* pMsg ){
    // change radio buttons selection if up/down keys are pressed
    // this MUST be changed if the layout of the dialog changes
    bool bProcessed =false;
    if(pMsg->message == WM_KEYDOWN){
        if(pMsg->wParam == VK_UP){
            pMsg->wParam = VK_LEFT;
        };

        if(pMsg->wParam == VK_DOWN){
            pMsg->wParam = VK_RIGHT;
        };
    }

    if(bProcessed)
        return TRUE;
    else
        return CDialog::PreTranslateMessage(pMsg);
}

HBRUSH CLoglevelSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
    #if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
    #endif

    return hbr;
}