/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

#include "stdafx.h"
#include "MailSettingsAdvanced.h"
#include "pim/ClientSettings.h"
#include "pim/Maincpp.h"

#include "comutil.h"
#include "tpcshell.h"
#include "pim/account.h"

// CMailSettingsAdvanced dialog
#define DEFAULT_USER "default@email"

IMPLEMENT_DYNAMIC(CMailSettingsAdvanced, CDialog)

CMailSettingsAdvanced::CMailSettingsAdvanced(CWnd* pParent /*=NULL*/)
	: CDialog(CMailSettingsAdvanced::IDD, pParent)
{

}

CMailSettingsAdvanced::~CMailSettingsAdvanced(){
  #if defined(WIN32_PLATFORM_PSPC)
    editEmail.Detach();
    editRemote.Detach();
    editName.Detach();
  #endif
}

void CMailSettingsAdvanced::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_MAIL_ADVANCED_EDIT_REMOTE, editRemote);
    DDX_Control(pDX, IDC_EMAIL_EDIT_DISPNAME, editName);
    DDX_Control(pDX, IDC_EMAIL_EDIT_EMAIL, editEmail);
}
BEGIN_MESSAGE_MAP(CMailSettingsAdvanced, CDialog)
#ifdef WIN32_PLATFORM_WFSP
    ON_MESSAGE(WM_HOTKEY,&CMailSettingsAdvanced::OnHotkey)
#endif
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


// CMailSettingsAdvanced message handlers

BOOL CMailSettingsAdvanced::OnInitDialog(){
    CString s1;
    wchar_t* accountName = new wchar_t[ACCOUNT_PARAM_SIZE];
    wchar_t* replyAddress = new wchar_t[ACCOUNT_PARAM_SIZE];

    s1.LoadString(IDS_ADVANCED_EMAIL); SetWindowText(s1);
    CDialog::OnInitDialog();

    // create menu bar
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL;
    #if defined WIN32_PLATFORM_WFSP
    #if (_WIN32_WCE < 0x500)
        dlgCommandBar.nToolBarId = IDR_MENUBAR_OKCANCEL;
    #endif
    #endif
    dlgCommandBar.hInstRes = ::AfxGetInstanceHandle();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    };

    editRemote.SetLimitText(EDIT_MAXLENGTH);

    LoadStringResources();

    // get account info
    SetCursor(LoadCursor(NULL, IDC_WAIT));
    HRESULT hr = getAccountInfo(accountName, replyAddress);   // this may take a while
    SetCursor(LoadCursor(NULL, IDC_ARROW));

    if (hr == -1) {
        delete [] accountName; delete [] replyAddress;
        AfxMessageBox(_T("Funambol email account not found."));
        CDialog::OnCancel();
        return FALSE;
    }

    CString sTemp1(accountName), sTemp2(replyAddress);
    SetDlgItemText(IDC_EMAIL_EDIT_DISPNAME,sTemp1);

    dispName = sTemp1; dispEmail = sTemp2;

    if (sTemp2 == DEFAULT_USER) {
        sTemp2 = "";
    }
    SetDlgItemText(IDC_EMAIL_EDIT_EMAIL, sTemp2);
    /*
    CString sTemp1(accountName), sTemp2(replyAddress);
    SetDlgItemText(IDC_EMAIL_EDIT_DISPNAME,sTemp1);
    SetDlgItemText(IDC_EMAIL_EDIT_EMAIL, sTemp2);
    */

    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));

    GetDlgItem(IDC_SETTMAIL_STATIC_REMONAME)->SetFont(&fontBold);
    GetDlgItem(IDC_EMAIL_STATIC_NAME)->SetFont(&fontBold);
    GetDlgItem(IDC_EMAIL_STATIC_EMAIL)->SetFont(&fontBold);

    // set ppc header
    #if defined(WIN32_PLATFORM_PSPC)
        s1.LoadString(IDS_ADVANCED_EMAIL); SetDlgItemText(IDC_HEADER_STATIC, s1);
        GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
    #endif

    // load settings from Registry
    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("mail");
    s1 = ssconf->getURI(); SetDlgItemText(IDC_MAIL_ADVANCED_EDIT_REMOTE,s1);

    // set a large int as current selection, so the cursor will be at end
    editRemote.SetSel(LARGE_INT ,LARGE_INT);
    editRemote.SetFocus();

    #ifdef WIN32_PLATFORM_WFSP
        // override back key
        LPARAM lparam = MAKELPARAM(SHMBOF_NODEFAULT | SHMBOF_NOTIFY,
        SHMBOF_NODEFAULT | SHMBOF_NOTIFY);
        ::SendMessage(dlgCommandBar.hwndMB, SHCMBM_OVERRIDEKEY, VK_TBACK, lparam);
    #endif

    if(accountName)
        delete [] accountName;
    if(replyAddress)
        delete [] replyAddress;

    return FALSE;
}

void CMailSettingsAdvanced::LoadStringResources(){
    CString s1;
    s1.LoadString(IDS_REMOTENAME); SetDlgItemText(IDC_SETTMAIL_STATIC_REMONAME,s1);
    s1.LoadString(IDS_ACCOUNT_DISPNAME); SetDlgItemText(IDC_EMAIL_STATIC_NAME,s1);
    s1.LoadString(IDS_ACCOUNT_EMAIL); SetDlgItemText(IDC_EMAIL_STATIC_EMAIL,s1);
}

afx_msg void CMailSettingsAdvanced::OnOK(){
    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("mail");
    CString s1,s2;
    _bstr_t bst;
    bool bSaveAccount = false;
    wchar_t* accountName = NULL;
    wchar_t* replyAddress = NULL;

    if(checkStartSync()){
        CDialog::OnOK();
    }

    GetDlgItemText(IDC_MAIL_ADVANCED_EDIT_REMOTE, s1);
    bst.Assign(s1.AllocSysString());
    ssconf->setURI((char*)bst);
    bst.Detach();

    // save mail account settings
    GetDlgItemText(IDC_EMAIL_EDIT_DISPNAME,s1);
    GetDlgItemText(IDC_EMAIL_EDIT_EMAIL,s2);
    /*
    if (s1.IsEmpty()) {
        AfxMessageBox(_T("The Display Name cannot be empty"));
        return;
    }
    if (s2.IsEmpty()) {
        AfxMessageBox(_T("The Email Address cannot be empty"));
        return;
    }
    */
    if (s1.IsEmpty()) {
        s1 = "";
    }
    if (s2.IsEmpty()) {
        s2 = DEFAULT_USER;
    }
    accountName = new wchar_t[ACCOUNT_PARAM_SIZE];
    replyAddress = new wchar_t[ACCOUNT_PARAM_SIZE];

    wcscpy(accountName, s1.GetBuffer( s1.GetLength() ));
    if(s1 != dispName)
        bSaveAccount = true;

    wcscpy( replyAddress, s2.GetBuffer( s2.GetLength() ));
    if(s2 != dispEmail)
        bSaveAccount = true;

    if( (accountName) && (replyAddress) && bSaveAccount) {
        SetCursor(LoadCursor(NULL, IDC_WAIT));
        HRESULT ret = -1;
        ret = modifyAccount(accountName, replyAddress);
        SetCursor(LoadCursor(NULL, IDC_ARROW));
    }

    if(accountName) {
        delete [] accountName; accountName = NULL;
    }
    if(replyAddress) {
        delete [] replyAddress; replyAddress = NULL;
    }

    CDialog::OnOK();
}

#ifdef WIN32_PLATFORM_WFSP
LRESULT CMailSettingsAdvanced::OnHotkey(WPARAM wParam, LPARAM lParam)
{
    if (HIWORD(lParam) == VK_TBACK)
        SHSendBackToFocusWindow(WM_HOTKEY, wParam, lParam);
    return 0;
}
#endif

HBRUSH CMailSettingsAdvanced::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
    #if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
    #endif

    return hbr;
}