/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

// TaskSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "TaskSettings.h"
#include "TaskSettingsAdvanced.h"

#include "ClientUtil.h"
#include "examples/config.h"
#include "base/fscapi.h"
#include "spds/constants.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/ManagementNode.h"

#include "pim/ClientSettings.h"
#include "pim/Maincpp.h"

// CTaskSettings dialog

IMPLEMENT_DYNAMIC(CTaskSettings, CDialog)

CTaskSettings::CTaskSettings(CWnd* pParent /*=NULL*/)
: CDialog(CTaskSettings::IDD, pParent)
{

}

CTaskSettings::~CTaskSettings()
{
}

void CTaskSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
#if defined(WIN32_PLATFORM_WFSP)
    DDX_Control(pDX, IDC_SETTTASK_SPIN1, spinSyncType );
#endif
    DDX_Control(pDX, IDC_TASK_SYNCTYPE, lstSyncType );
    DDX_Control(pDX, IDC_TASK_LINK_ADVANCED, linkAdvanced);
}


BEGIN_MESSAGE_MAP(CTaskSettings, CDialog)
    ON_BN_CLICKED(IDC_TASK_LINK_ADVANCED, &CTaskSettings::OnBnClickedTaskLinkAdvanced)
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


// CTaskSettings message handlers

BOOL CTaskSettings::OnInitDialog()
{
    CString s1;
    s1.LoadString(IDS_TASKS_SETTINGS); SetWindowText(s1);
    CDialog::OnInitDialog();

#if defined(WIN32_PLATFORM_WFSP)
    spinSyncType.SetRange(4,0);	spinSyncType.SetBuddy(&lstSyncType);
#endif

    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL;
#if defined WIN32_PLATFORM_WFSP
#if (_WIN32_WCE < 0x500)
    dlgCommandBar.nToolBarId = IDR_MENUBAR_OKCANCEL;
#endif
#endif
    dlgCommandBar.hInstRes = ::AfxGetInstanceHandle();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    }

    // read string resources
    s1.LoadString(IDS_SETTCOMM_SYNCTYPE1);	lstSyncType.AddString(s1);
    s1.LoadString(IDS_SETTCOMM_SYNCTYPE2);	lstSyncType.AddString(s1);
    s1.LoadString(IDS_SETTCOMM_SYNCTYPE3);	lstSyncType.AddString(s1);


    s1.LoadString(IDS_SYNCTYPE); SetDlgItemText(IDC_SETTTASK_STATIC_SYNCTYPE,s1);
    s1.LoadString(IDS_ADVANCED_SETTINGS); linkAdvanced.setText(s1);

    // set font
    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));
    GetDlgItem(IDC_SETTTASK_STATIC_SYNCTYPE)->SetFont(&fontBold);

    // set ppc header
#if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(IDS_TASKS_SETTINGS); SetDlgItemText(IDC_HEADER_STATIC, s1);
    GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
#endif

    // read from the registry
    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("task");
    int pos=0;

    pos = SynctypeReg2UI(ssconf->getSync());
    lstSyncType.SetCurSel(pos);

    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    lstSyncType.SetFocus();
    return FALSE; //SET focus
}

void CTaskSettings::OnOK()
{
    // save values in registry
    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("task");
    CString s1; int pos=0;

    if(checkStartSync()){
        CDialog::OnOK();
    };

    pos = lstSyncType.GetCurSel();
    if(strcmp(SynctypeUI2Reg(pos),ssconf->getSync()) != 0){
        ssconf->setSync(SynctypeUI2Reg(pos));

        getRegConfig()->setDirty(CS_DIRTY_SOURCE_TASKS);
        getRegConfig()->saveDirty();
    }

    CDialog::OnOK();
}

void CTaskSettings::OnBnClickedTaskLinkAdvanced()
{
    CTaskSettingsAdvanced wndTaskAdvanced;

    INT_PTR result = wndTaskAdvanced.DoModal();
}

// capture pressing of 'Advanced' link
BOOL CTaskSettings::PreTranslateMessage(MSG* pMsg){
    bool bProcessed =false;
    if(pMsg->message == WM_KEYDOWN){
        if(pMsg->wParam == VK_RETURN){
            if(GetFocus()->GetSafeHwnd() == linkAdvanced.GetSafeHwnd()){
                bProcessed = true;
                OnBnClickedTaskLinkAdvanced();
            }

        }
    };

    if(bProcessed)
        return TRUE;
    else
        return CDialog::PreTranslateMessage(pMsg);
}

HBRUSH CTaskSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
#if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
#endif

    return hbr;
}