/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

// TaskSettingsAdvanced.cpp : implementation file
//

#include "stdafx.h"
#include "TaskSettingsAdvanced.h"
#include "comutil.h"
#include "pim/ClientSettings.h"
#include "pim/Maincpp.h"
#include "tpcshell.h"

// CTaskSettingsAdvanced dialog

IMPLEMENT_DYNAMIC(CTaskSettingsAdvanced, CDialog)

CTaskSettingsAdvanced::CTaskSettingsAdvanced(CWnd* pParent /*=NULL*/)
	: CDialog(CTaskSettingsAdvanced::IDD, pParent)
{

}

CTaskSettingsAdvanced::~CTaskSettingsAdvanced(){
  #if defined(WIN32_PLATFORM_PSPC)
    editRemote.Detach();
  #endif
}

void CTaskSettingsAdvanced::DoDataExchange(CDataExchange* pDX)
{
    DDX_Control(pDX, IDC_SETTTASK_EDIT_REMOTE, editRemote);
	CDialog::DoDataExchange(pDX);
}

BOOL CTaskSettingsAdvanced::OnInitDialog(){
    CString s1;
    s1.LoadString(IDS_ADVANCED_TASKS); SetWindowText(s1);
    CDialog::OnInitDialog();

    editRemote.SetLimitText(EDIT_MAXLENGTH);

    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL;
    #if defined WIN32_PLATFORM_WFSP
    #if (_WIN32_WCE < 0x500)
        dlgCommandBar.nToolBarId = IDR_MENUBAR_OKCANCEL;
    #endif
    #endif
    dlgCommandBar.hInstRes = ::AfxGetInstanceHandle();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return FALSE;
    };

    LoadStringResources();

    LOGFONT lf;
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfWeight = FW_BOLD;
    lf.lfHeight +=FONT_HEIGHT_OFFSET;
    VERIFY(fontBold.CreateFontIndirect(&lf));
    GetDlgItem(IDC_SETTTASK_STATIC_REMONAME)->SetFont(&fontBold);
    GetDlgItem(IDC_SETTTASK_STATIC_SYNCFORM)->SetFont(&fontBold);

    // set ppc header
    #if defined(WIN32_PLATFORM_PSPC)
        s1.LoadString(IDS_ADVANCED_TASKS); SetDlgItemText(IDC_HEADER_STATIC, s1);
        GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
    #endif

    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("task");
    s1 = ssconf->getURI(); SetDlgItemText(IDC_SETTTASK_EDIT_REMOTE,s1);

    editRemote.SetSel(LARGE_INT ,LARGE_INT);
    editRemote.SetFocus();

    #ifdef WIN32_PLATFORM_WFSP
        // override back key
        LPARAM lparam = MAKELPARAM(SHMBOF_NODEFAULT | SHMBOF_NOTIFY,
            SHMBOF_NODEFAULT | SHMBOF_NOTIFY);
        ::SendMessage(dlgCommandBar.hwndMB, SHCMBM_OVERRIDEKEY, VK_TBACK, lparam);
    #endif

    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    return FALSE;
}

void CTaskSettingsAdvanced::OnOK(){

    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("task");
    CString s1;
    _bstr_t bst;

    if(checkStartSync()){
        CDialog::OnOK();
    };

    GetDlgItemText(IDC_SETTTASK_EDIT_REMOTE, s1);
    bst.Assign(s1.AllocSysString());

    if(strcmp((char*)bst,ssconf->getURI()) != 0){

        ssconf->setURI((char*)bst);
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_TASKS);
        getRegConfig()->saveDirty();
    }

    bst.Detach();
    CDialog::OnOK();
}

BEGIN_MESSAGE_MAP(CTaskSettingsAdvanced, CDialog)
#ifdef WIN32_PLATFORM_WFSP
    ON_MESSAGE(WM_HOTKEY,&CTaskSettingsAdvanced::OnHotkey)
#endif
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()


// CTaskSettingsAdvanced message handlers
void CTaskSettingsAdvanced::LoadStringResources(){
    CString s1;
    s1.LoadString(IDS_REMOTENAME); SetDlgItemText(IDC_SETTTASK_STATIC_REMONAME,s1);
    s1.LoadString(IDS_SETTTASK_SYNCFORMAT); SetDlgItemText(IDC_SETTTASK_STATIC_SYNCFORM,s1);
    s1.LoadString(IDS_SETTTASK_FORMAT1); SetDlgItemText(IDC_TAKS_STATIC_SIF,s1);
}

#ifdef WIN32_PLATFORM_WFSP
LRESULT CTaskSettingsAdvanced::OnHotkey(WPARAM wParam, LPARAM lParam)
{
    if (HIWORD(lParam) == VK_TBACK)
        SHSendBackToFocusWindow(WM_HOTKEY, wParam, lParam);
    return 0;
}
#endif

HBRUSH CTaskSettingsAdvanced::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
    #if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
    #endif

    return hbr;
}