/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

//
// @author Marco Magistrali
//
#include "pim/ContactBuilder.h"
#include "pim/Utils.h"
#include "base/quoted-printable.h"
#include "base/util/utils.h"
#include "base/util/WString.h"

/*
* Method to populate the contact XML structure.
* It get every property name and value and build the XML structure to exchange with server.
*
* @param contactStringItem  : the string that will contain the xml structure
* @param pAppointment       : the contact object that contain every value
*
*/

void populateContactStringItem(wstring &contactStringItem, IContact *pContact) {

    BSTR element = NULL;
    DATE date = NULL;
    wchar_t tempDate[50];
    wstring element_s;

    contactStringItem = TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    contactStringItem += TEXT("<contact>");

    pContact->get_FirstName(&element);
    element_s.clear();
    element_s = wstring(element);
    encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<FirstName>") + element_s + TEXT("</FirstName>");
    SysFreeString(element);

    pContact->get_LastName(&element);
    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<LastName>") + element_s + TEXT("</LastName>");
    SysFreeString(element);

    pContact->get_CompanyName(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<CompanyName>") + element_s + TEXT("</CompanyName>");
    SysFreeString(element);

    pContact->get_BusinessFaxNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessFaxNumber>") + element_s + TEXT("</BusinessFaxNumber>");
    SysFreeString(element);

    pContact->get_Department(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Department>") + element_s + TEXT("</Department>");
    SysFreeString(element);

    pContact->get_Email1Address(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Email1Address>") + element_s + TEXT("</Email1Address>");
    SysFreeString(element);

    pContact->get_MobileTelephoneNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<MobileTelephoneNumber>") + element_s + TEXT("</MobileTelephoneNumber>");
    SysFreeString(element);

    pContact->get_OfficeLocation(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<OfficeLocation>") + element_s + TEXT("</OfficeLocation>");
    SysFreeString(element);

    pContact->get_PagerNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<PagerNumber>") + element_s + TEXT("</PagerNumber>");
    SysFreeString(element);

    pContact->get_BusinessTelephoneNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessTelephoneNumber>") + element_s + TEXT("</BusinessTelephoneNumber>");
    SysFreeString(element);

    pContact->get_JobTitle(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<JobTitle>") + element_s + TEXT("</JobTitle>");
    SysFreeString(element);

    pContact->get_HomeTelephoneNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeTelephoneNumber>") + element_s + TEXT("</HomeTelephoneNumber>");
    SysFreeString(element);

    pContact->get_Email2Address(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Email2Address>") + element_s + TEXT("</Email2Address>");
    SysFreeString(element);

    pContact->get_Spouse(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Spouse>") + element_s + TEXT("</Spouse>");
    SysFreeString(element);

    pContact->get_Email3Address(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Email3Address>") + element_s + TEXT("</Email3Address>");
    SysFreeString(element);

    pContact->get_Home2TelephoneNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Home2TelephoneNumber>") + element_s + TEXT("</Home2TelephoneNumber>");
    SysFreeString(element);

    pContact->get_HomeFaxNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeFaxNumber>") + element_s + TEXT("</HomeFaxNumber>");
    SysFreeString(element);

    pContact->get_CarTelephoneNumber(&element);

    element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<CarTelephoneNumber>") + element_s + TEXT("</CarTelephoneNumber>");
    SysFreeString(element);

    pContact->get_AssistantName(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<AssistantName>") + element_s + TEXT("</AssistantName>");
    SysFreeString(element);

    pContact->get_AssistantTelephoneNumber(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<AssistantTelephoneNumber>") + element_s + TEXT("</AssistantTelephoneNumber>");
    SysFreeString(element);

    pContact->get_Children(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Children>") + element_s + TEXT("</Children>");
    SysFreeString(element);

    pContact->get_Categories(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Categories>") + element_s + TEXT("</Categories>");
    SysFreeString(element);

    pContact->get_WebPage(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<WebPage>") + element_s + TEXT("</WebPage>");
    SysFreeString(element);

    pContact->get_Business2TelephoneNumber(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Business2TelephoneNumber>") + element_s + TEXT("</Business2TelephoneNumber>");
    SysFreeString(element);

    pContact->get_RadioTelephoneNumber(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<RadioTelephoneNumber>") + element_s + TEXT("</RadioTelephoneNumber>");
    SysFreeString(element);

    pContact->get_FileAs(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<FileAs>") + element_s + TEXT("</FileAs>");
    SysFreeString(element);

    pContact->get_YomiCompanyName(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<YomiCompanyName>") + element_s + TEXT("</YomiCompanyName>");
    SysFreeString(element);

    pContact->get_YomiFirstName(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<YomiFirstName>") + element_s + TEXT("</YomiFirstName>");
    SysFreeString(element);

    pContact->get_YomiLastName(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<YomiLastName>") + element_s + TEXT("</YomiLastName>");
    SysFreeString(element);

    pContact->get_Title(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Title>") + element_s + TEXT("</Title>");
    SysFreeString(element);

    pContact->get_MiddleName(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<MiddleName>") + element_s + TEXT("</MiddleName>");
    SysFreeString(element);

    pContact->get_Suffix(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Suffix>") + element_s + TEXT("</Suffix>");
    SysFreeString(element);

    pContact->get_HomeAddressStreet(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeAddressStreet>") + element_s + TEXT("</HomeAddressStreet>");
    SysFreeString(element);

    pContact->get_HomeAddressCity(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeAddressCity>") + element_s + TEXT("</HomeAddressCity>");
    SysFreeString(element);

    pContact->get_HomeAddressState(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeAddressState>") + element_s + TEXT("</HomeAddressState>");
    SysFreeString(element);

    pContact->get_HomeAddressPostalCode(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeAddressPostalCode>") + element_s + TEXT("</HomeAddressPostalCode>");
    SysFreeString(element);

    pContact->get_HomeAddressCountry(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<HomeAddressCountry>") + element_s + TEXT("</HomeAddressCountry>");
    SysFreeString(element);

    pContact->get_OtherAddressStreet(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<OtherAddressStreet>") + element_s + TEXT("</OtherAddressStreet>");
    SysFreeString(element);

    pContact->get_OtherAddressCity(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<OtherAddressCity>") + element_s + TEXT("</OtherAddressCity>");
    SysFreeString(element);

    pContact->get_OtherAddressPostalCode(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<OtherAddressPostalCode>") + element_s + TEXT("</OtherAddressPostalCode>");
    SysFreeString(element);

    pContact->get_OtherAddressCountry(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<OtherAddressCountry>") + element_s + TEXT("</OtherAddressCountry>");
    SysFreeString(element);

    pContact->get_OtherAddressState(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<OtherAddressState>") + element_s + TEXT("</OtherAddressState>");
    SysFreeString(element);

    pContact->get_BusinessAddressStreet(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessAddressStreet>") + element_s + TEXT("</BusinessAddressStreet>");
    SysFreeString(element);

    pContact->get_BusinessAddressCity(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessAddressCity>") + element_s + TEXT("</BusinessAddressCity>");
    SysFreeString(element);

    pContact->get_BusinessAddressState(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessAddressState>") + element_s + TEXT("</BusinessAddressState>");
    SysFreeString(element);

    pContact->get_BusinessAddressPostalCode(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessAddressPostalCode>") + element_s + TEXT("</BusinessAddressPostalCode>");
    SysFreeString(element);

    pContact->get_BusinessAddressCountry(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<BusinessAddressCountry>") + element_s + TEXT("</BusinessAddressCountry>");
    SysFreeString(element);

    pContact->get_Body(&element); element_s.clear();
    element_s = wstring(element); encodeSpecialChar(element_s);
    contactStringItem  += TEXT("<Body>") + element_s + TEXT("</Body>");
    SysFreeString(element);

    pContact->get_Birthday(&date);

    wsprintf(tempDate, TEXT(""));

    double toCompare = MAX_DATE_DOUBLE;

    if (date != toCompare) { // MAX_DATE_DOUBLE is set to 949999.00000, the nonsense value when in ppc date is not set...
        doubleToSystemTimeBirthday(tempDate, date);
        //doubleToSystemTimeHourZero(tempDate, date, TRUE);
        // doubleToSystemTime(tempDate, date);
    }
    //contactStringItem +=  TEXT("<Birthday>") + tempDate + TEXT("</Birthday>");
    contactStringItem +=  TEXT("<Birthday>");  contactStringItem += tempDate; contactStringItem += TEXT("</Birthday>");

    pContact->get_Anniversary(&date);
    wsprintf(tempDate, TEXT(""));
    if (date != toCompare) {
        doubleToSystemTimeBirthday(tempDate, date);
        // doubleToSystemTimeHourZero(tempDate, date, TRUE);
        // doubleToSystemTime(tempDate, date);
    }
    //contactStringItem +=  TEXT("<Anniversary>") + tempDate + TEXT("</Anniversary>");

    contactStringItem +=  TEXT("<Anniversary>"); contactStringItem += tempDate; contactStringItem += TEXT("</Anniversary>");

    contactStringItem +=  TEXT("</contact>");

}

/*
* Method that complete an contact object retrieving property value by an XML structure.
* It parse the XML structure, get the every property value using its name and set it into the object.
*
* @param pContact     : the contact object that contain every value
* @param ptrData      : the variable that contain the xml structure
*/

void completeContact(IContact *pContact, wchar_t * ptrData) {
        wchar_t* dummy = NULL;
        wchar_t* dummyEvent = NULL;
        wchar_t localTemp [DIM_LOCAL_TEMP];
        wstring dummyEvent_s;

        double doubleDummy = MAX_DATE_DOUBLE;

        // complete fields for updating
        // BSTR
        dummyEvent = getElementContent(ptrData, TEXT ("FirstName"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_FirstName ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent; dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("LastName"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_LastName ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("CompanyName"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_CompanyName ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("BusinessFaxNumber"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_BusinessFaxNumber ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Department"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_Department ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Email1Address"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_Email1Address ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("MobileTelephoneNumber"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_MobileTelephoneNumber ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("OfficeLocation"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_OfficeLocation ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("PagerNumber"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_PagerNumber ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("BusinessTelephoneNumber"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_BusinessTelephoneNumber ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("JobTitle"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_JobTitle ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("HomeTelephoneNumber"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_HomeTelephoneNumber  ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Email2Address"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_Email2Address ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Spouse"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_Spouse ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Email3Address"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_Email3Address ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Home2TelephoneNumber"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_Home2TelephoneNumber ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("HomeFaxNumber"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_HomeFaxNumber ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("CarTelephoneNumber"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_CarTelephoneNumber ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("AssistantName"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_AssistantName ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("AssistantTelephoneNumber"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_AssistantTelephoneNumber ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Children"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_Children ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Categories"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_Categories ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("WebPage"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_WebPage ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Business2TelephoneNumber"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_Business2TelephoneNumber ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("RadioTelephoneNumber"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_RadioTelephoneNumber ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("FileAs"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_FileAs ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("YomiCompanyName"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_YomiCompanyName ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("YomiFirstName"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_YomiFirstName ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("YomiLastName"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_YomiLastName ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Title"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_Title ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("MiddleName"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_MiddleName ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Suffix"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_Suffix ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("HomeAddressStreet"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            convertNewlines(dummyEvent_s); // BUG #1562
            pContact->put_HomeAddressStreet ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("HomeAddressState"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_HomeAddressState ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("HomeAddressCity"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_HomeAddressCity ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("HomeAddressPostalCode"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_HomeAddressPostalCode ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("HomeAddressCountry"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_HomeAddressCountry ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("OtherAddressStreet"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            convertNewlines(dummyEvent_s); // BUG #1562
            pContact->put_OtherAddressStreet ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("OtherAddressCity"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_OtherAddressCity ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("OtherAddressState"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_OtherAddressState ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("OtherAddressPostalCode"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_OtherAddressPostalCode ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("OtherAddressCountry"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_OtherAddressCountry ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("BusinessAddressStreet"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            convertNewlines(dummyEvent_s); // BUG #1562
            pContact->put_BusinessAddressStreet ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("BusinessAddressCity"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_BusinessAddressCity ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("BusinessAddressState"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_BusinessAddressState ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("BusinessAddressPostalCode"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_BusinessAddressPostalCode ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("BusinessAddressCountry"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            pContact->put_BusinessAddressCountry ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Body"), NULL);
        if (dummyEvent != NULL) {
            dummyEvent_s = wstring(dummyEvent); decodeSpecialChar(dummyEvent_s);
            convertNewlines(dummyEvent_s); // BUG #1562
            pContact->put_Body ((wchar_t*)dummyEvent_s.data());
            delete [] dummyEvent;dummyEvent=NULL;
        }

        // tipo DATE
        dummyEvent = getElementContent(ptrData, TEXT ("Birthday"), NULL);
        if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
            pContact->put_Birthday(doubleDummy);
        else {
            DATE birth;
            //systemTimeToDouble(dummyEvent, &birth, NULL);
            systemTimeToDoubleBirthday(dummyEvent, &birth);
            wsprintf(localTemp, TEXT("%f"), birth);

            pContact->put_Birthday(birth);
            delete [] dummyEvent;
        }
        if (dummyEvent) {delete [] dummyEvent; dummyEvent = NULL; }

        dummyEvent = getElementContent(ptrData, TEXT ("Anniversary"), NULL);
        if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
            pContact->put_Anniversary(doubleDummy);
        else {

            DATE anny;
            //systemTimeToDouble(dummyEvent, &anny, NULL);
            systemTimeToDoubleBirthday(dummyEvent, &anny);
            pContact->put_Anniversary(anny);
            delete [] dummyEvent;
        }
        if (dummyEvent) {delete [] dummyEvent; dummyEvent = NULL; }

}

/*
* The method calculate the CRC hash value for the contact item.
* It create the XML structure and calculate the crc value.
* Use to compare for understand the modified value!
*
* @param pContact : the contact object that contain every value
*/

long calculateContactHash (IContact* pContact) {

    wstring p;

    populateContactStringItem(p, pContact);
    long l = stringCrc32(p.data());

    p.clear();
    return l;

}
VObject* ContactToVObject(IContact *pContact) {

    VObject* vo = new VObject();
    VProperty* vp    = NULL;
    bool value_found = false;

    vp = new VProperty(TEXT("BEGIN"), TEXT("VCARD"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    vp = new VProperty(TEXT("VERSION"), TEXT("2.1"));
    vo->addProperty(vp);
    delete vp; vp = NULL;


    BSTR element = NULL;
    DATE date = NULL;

    // ------- Name -------
    vp = new VProperty(TEXT("N"));

    pContact->get_LastName(&element);
    vp->addValue(element);

    pContact->get_FirstName(&element);
    vp->addValue(element);

    pContact->get_MiddleName(&element);
    vp->addValue(element);

    pContact->get_Title(&element);
    vp->addValue(element);

   pContact->get_Suffix(&element);
    vp->addValue(element);

    vo->addProperty(vp);
    delete vp; vp = NULL;


    // Formatted Name FN = <FileAs>
    pContact->get_FileAs(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("FN"), element);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }


// END NAME

    wchar_t tempDate[50];
    wsprintf(tempDate, TEXT(""));

    double toCompare = MAX_DATE_DOUBLE;

    pContact->get_Birthday(&date);
    if(date) {
        if (date != toCompare) { // MAX_DATE_DOUBLE is set to 949999.00000, the nonsense value when in ppc date is not set...
            doubleToSystemTimeBirthday(tempDate, date);
        }
        vp = new VProperty(TEXT("BDAY"), tempDate);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    //Anniversary has no corespondent in the VCARD

    pContact->get_BusinessFaxNumber(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("TEL"), element);
        vp->addParameter(TEXT("WORK"), NULL);
        vp->addParameter(TEXT("FAX"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    pContact->get_CompanyName(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("ORG"), element);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    //Department has no corespondent in the VCARD

    pContact->get_Email1Address(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("EMAIL"), element);
        vp->addParameter(TEXT("INTERNET"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    pContact->get_Email2Address(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("EMAIL"), element);
        vp->addParameter(TEXT("INTERNET"), NULL);
        vp->addParameter(TEXT("HOME"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    pContact->get_Email3Address(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("EMAIL"), element);
        vp->addParameter(TEXT("INTERNET"), NULL);
        vp->addParameter(TEXT("WORK"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;

    }

    pContact->get_MobileTelephoneNumber(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("TEL"), element);
        vp->addParameter(TEXT("CELL"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;

    }

    //OfficeLocation has no corespondent in the VCARD

    pContact->get_PagerNumber(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("TEL"), element);
        vp->addParameter(TEXT("PAGER"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;

    }

    pContact->get_BusinessTelephoneNumber(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("TEL"), element);
        vp->addParameter(TEXT("WORK"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    pContact->get_JobTitle(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("TITLE"), element);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    pContact->get_HomeTelephoneNumber(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("TEL"), element);
        vp->addParameter(TEXT("HOME"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    //Spouse has no corespondent in the VCARD

    pContact->get_Home2TelephoneNumber(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("TEL"), element);
        vp->addParameter(TEXT("HOME"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    pContact->get_CarTelephoneNumber(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("TEL"), element);
        vp->addParameter(TEXT("CAR"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    pContact->get_HomeFaxNumber(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("TEL"), element);
        vp->addParameter(TEXT("HOME"), NULL);
        vp->addParameter(TEXT("FAX"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    //get_AssistantName X
    //get_AssistantTelephoneNumber X

    //Children has no corespondent in the VCARD

    pContact->get_Categories(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("CATEGORIES"), element);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    pContact->get_WebPage(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("URL"), element);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    pContact->get_Business2TelephoneNumber(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("TEL"), element);
        vp->addParameter(TEXT("WORK"), NULL);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    //RadioTelephoneNumber has no corespondent in the VCARD

    //Yomi properties have no corespondent in the VCARD

        // ----- Address HOME -----

    vp = new VProperty(TEXT("ADR"));
    vp->addParameter(TEXT("HOME"), NULL);
    value_found = false;

    // First two values are empty:
    // "Post Office Address" and "Extended Address" not present in SIF
    vp->addValue(TEXT(""));
    vp->addValue(TEXT(""));

    pContact->get_HomeAddressStreet(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    pContact->get_HomeAddressCity(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    pContact->get_HomeAddressState(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    pContact->get_HomeAddressPostalCode(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    pContact->get_HomeAddressCountry(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    if(value_found) {
        vo->addProperty(vp);
    }
    delete vp; vp = NULL;


    // END ADDRESS HOME

    vp = new VProperty(TEXT("ADR"));
    value_found = false;

    // First two values are empty:
    // "Post Office Address" and "Extended Address" not present in SIF
    vp->addValue(TEXT(""));
    vp->addValue(TEXT(""));

    pContact->get_OtherAddressStreet(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    pContact->get_OtherAddressCity(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    pContact->get_OtherAddressState(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    pContact->get_OtherAddressPostalCode(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    pContact->get_OtherAddressCountry(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    if(value_found) {
        vo->addProperty(vp);
    }
    delete vp; vp = NULL;

    // END OTHER

    vp = new VProperty(TEXT("ADR"));
    vp->addParameter(TEXT("WORK"), NULL);
    value_found = false;

    // First two values are empty:
    // "Post Office Address" and "Extended Address" not present in SIF
    vp->addValue(TEXT(""));
    vp->addValue(TEXT(""));

    pContact->get_BusinessAddressStreet(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    pContact->get_BusinessAddressCity(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    pContact->get_BusinessAddressState(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    pContact->get_BusinessAddressPostalCode(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    pContact->get_BusinessAddressCountry(&element);
    if(wcscmp(element, TEXT("")))
        value_found = true;
    vp->addValue(element);


    if(value_found) {
        vo->addProperty(vp);
    }
    delete vp; vp = NULL;

   // END BUSINESS

    pContact->get_Body(&element);
    if(wcscmp(element, TEXT(""))) {
        vp = new VProperty(TEXT("NOTE"), element);
        vo->addProperty(vp);
        delete vp; vp = NULL;
    }

    vp = new VProperty(TEXT("END"), TEXT("VCARD"));
    vo->addProperty(vp);
    delete vp; vp = NULL;

    return vo;

}

void VObjectToContact(IContact *pContact, VObject *vo) {

    int businessTel = 0;
    int homeTel        = 0;
    int email        = 0;

    wchar_t* name    = NULL;
    wchar_t* element = NULL;

    for(int i = 0; i < vo->propertiesCount(); i++) {

        name    = vo->getProperty(i)->getName();
        element = vo->getProperty(i)->getValue(0);

        if(!wcscmp(name, TEXT("N"))) {
            if(vo->getProperty(i)->getPropComponent(1))
                pContact->put_LastName(vo->getProperty(i)->getPropComponent(1));
            if(vo->getProperty(i)->getPropComponent(2))
                pContact->put_FirstName(vo->getProperty(i)->getPropComponent(2));
            if(vo->getProperty(i)->getPropComponent(3))
                pContact->put_MiddleName(vo->getProperty(i)->getPropComponent(3));
            if(vo->getProperty(i)->getPropComponent(4))
                pContact->put_Title(vo->getProperty(i)->getPropComponent(4));
            if(vo->getProperty(i)->getPropComponent(5))
                pContact->put_Suffix(vo->getProperty(i)->getPropComponent(5));
        }
        else if(!wcscmp(name, TEXT("FN"))) {
            pContact->put_FileAs(element);
        }
        else if(!wcscmp(name, TEXT("BDAY"))) {
            DATE birth;
            systemTimeToDoubleBirthday(element, &birth);
            pContact->put_Birthday(birth);
        }

        else if(!wcscmp(name, TEXT("TEL")) &&
                vo->getProperty(i)->containsParameter(TEXT("WORK"))) {
            if (vo->getProperty(i)->containsParameter(TEXT("FAX")))
                pContact->put_BusinessFaxNumber(element);
            else if (vo->getProperty(i)->containsParameter(TEXT("CELL")))
                ;
            else {
                if(businessTel == 0) {
                    pContact->put_BusinessTelephoneNumber(element);
                    businessTel++;
                }
                else {
                    pContact->put_Business2TelephoneNumber(element);
                }
            }
        }

        else if(!wcscmp(name, TEXT("ORG")))
            pContact->put_CompanyName(element);

        else if(!wcscmp(name, TEXT("EMAIL"))) {

            if (vo->getProperty(i)->containsParameter(L"HOME"))  {
                pContact->put_Email2Address(element);
            } else if (vo->getProperty(i)->containsParameter(L"WORK"))  {
                pContact->put_Email3Address(element);
            } else {
                pContact->put_Email1Address(element);
            }
            /*
            if(email == 0) {
                pContact->put_Email1Address(element);
                email++;
            }
            else if(email == 1) {
                email++;
                pContact->put_Email2Address(element);
            }
            else
                pContact->put_Email3Address(element);
            */
        }

        else if(!wcscmp(name, TEXT("TEL")) &&
            vo->getProperty(i)->containsParameter(TEXT("CELL")))
            pContact->put_MobileTelephoneNumber(element);

        else if(!wcscmp(name, TEXT("TEL")) &&
            vo->getProperty(i)->containsParameter(TEXT("PAGER")))
            pContact->put_PagerNumber(element);

        else if(!wcscmp(name, TEXT("TITLE")))
            pContact->put_JobTitle(element);

        else if(!wcscmp(name, TEXT("TEL")) &&
            vo->getProperty(i)->containsParameter(TEXT("HOME"))) {
            if(vo->getProperty(i)->containsParameter(TEXT("FAX")))
                pContact->put_HomeFaxNumber(element);
            else if(vo->getProperty(i)->containsParameter(TEXT("CELL")))
                ;
            else if(homeTel == 0) {
                pContact->put_HomeTelephoneNumber(element);
                homeTel++;
                }
            else {
                pContact->put_Home2TelephoneNumber(element);
            }
        }

        else if(!wcscmp(name, TEXT("TEL")) &&
            vo->getProperty(i)->containsParameter(TEXT("CAR")))
            pContact->put_CarTelephoneNumber(element);

        else if(!wcscmp(name, TEXT("CATEGORIES")))
            pContact->put_Categories(element);

        else if(!wcscmp(name, TEXT("URL")))
            pContact->put_WebPage(element);

        else if(!wcscmp(name, TEXT("NOTE")))
            pContact->put_Body(element);

        else if(!wcscmp(name, TEXT("ADR"))) {
            if(vo->getProperty(i)->containsParameter(TEXT("HOME"))) {
                if(vo->getProperty(i)->getPropComponent(3))
                    pContact->put_HomeAddressStreet(vo->getProperty(i)->getPropComponent(3));
                if(vo->getProperty(i)->getPropComponent(4))
                    pContact->put_HomeAddressCity(vo->getProperty(i)->getPropComponent(4));
                if(vo->getProperty(i)->getPropComponent(5))
                    pContact->put_HomeAddressState(vo->getProperty(i)->getPropComponent(5));
                if(vo->getProperty(i)->getPropComponent(6))
                    pContact->put_HomeAddressPostalCode(vo->getProperty(i)->getPropComponent(6));
                if(vo->getProperty(i)->getPropComponent(7))
                    pContact->put_HomeAddressCountry(vo->getProperty(i)->getPropComponent(7));
            }
            else if(vo->getProperty(i)->containsParameter(TEXT("WORK"))) {
                if(vo->getProperty(i)->getPropComponent(3))
                    pContact->put_BusinessAddressStreet(vo->getProperty(i)->getPropComponent(3));
                if(vo->getProperty(i)->getPropComponent(4))
                    pContact->put_BusinessAddressCity(vo->getProperty(i)->getPropComponent(4));
                if(vo->getProperty(i)->getPropComponent(5))
                    pContact->put_BusinessAddressState(vo->getProperty(i)->getPropComponent(5));
                if(vo->getProperty(i)->getPropComponent(6))
                    pContact->put_BusinessAddressPostalCode(vo->getProperty(i)->getPropComponent(6));
                if(vo->getProperty(i)->getPropComponent(7))
                    pContact->put_BusinessAddressCountry(vo->getProperty(i)->getPropComponent(7));
            }
            else {
                if(vo->getProperty(i)->getPropComponent(3))
                    pContact->put_OtherAddressStreet(vo->getProperty(i)->getPropComponent(3));
                if(vo->getProperty(i)->getPropComponent(4))
                    pContact->put_OtherAddressCity(vo->getProperty(i)->getPropComponent(4));
                if(vo->getProperty(i)->getPropComponent(5))
                    pContact->put_OtherAddressState(vo->getProperty(i)->getPropComponent(5));
                if(vo->getProperty(i)->getPropComponent(6))
                    pContact->put_OtherAddressPostalCode(vo->getProperty(i)->getPropComponent(6));
                if(vo->getProperty(i)->getPropComponent(7))
                    pContact->put_OtherAddressCountry(vo->getProperty(i)->getPropComponent(7));
            }
        }
    }
}

