/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

#include "spdm/Constants.h"

#include "pim/SettingFunctions.h"
#include <winioctl.h>
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "base/util/utils.h"
#include <uniqueid.h>
#include <commctrl.h>
#include <aygshell.h>
#include <sipapi.h>
#include <cfgmgrapi.h>

#include <string>

#define MAX_DEVID 50


#define IOCTL_HAL_GET_DEVICEID CTL_CODE(FILE_DEVICE_HAL, 21, METHOD_BUFFERED, FILE_ANY_ACCESS)

extern "C" __declspec(dllimport) BOOL KernelIoControl(
                                                        DWORD dwIoControlCode,
                                                        LPVOID lpInBuf,
                                                        DWORD nInBufSize,
                                                        LPVOID lpOutBuf,
                                                        DWORD nOutBufSize,
                                                        LPDWORD lpBytesReturned
);

/*
* Get the unique Device Identifier from PPC 2002 - 2003
* see more at http://www.pocketpcdn.com/articles/serial_number2002.html
*
* @param deviceId     : the retrieved device id
*/

wstring GetSerialNumberFromKernelIoControl() {

    DWORD dwOutBytes = 0;
    const int nBuffSize = 2048;
    byte arrOutBuff[nBuffSize];
    BOOL bRes = TRUE;

    wstring deviceId;

    bRes = KernelIoControl(IOCTL_HAL_GET_DEVICEID, 0, 0, arrOutBuff, nBuffSize, &dwOutBytes);

    if(!bRes) {
        /*
        * Repeat function call GetSerialNumberFromKernelIoControl to fix
        * retrieve serial number from Asus MyPal 620.
        * It's seems this function doesn't work properly if the function
        * is called only one time.
        */
        bRes = KernelIoControl(IOCTL_HAL_GET_DEVICEID, 0, 0, arrOutBuff, nBuffSize, &dwOutBytes);
    }

    if (bRes) {

        try {
            wchar_t tmp [10];
            for (unsigned int i = 0; i<dwOutBytes; i++) {
                wsprintf(tmp, TEXT("%02X"), arrOutBuff[i]);
                deviceId += tmp;
            }

            deviceId.replace(0, 40,_T(""));
            deviceId.replace(2, 3,_T(""));
            deviceId.replace(11, 16,_T(""));
            deviceId.replace(17, 4,_T(""));

        } catch (std::exception) {

            DEVICE_ID *deviceIDInfo = (DEVICE_ID *)arrOutBuff;

            if (deviceIDInfo != NULL) {

                /* Convert PresetID into the string */
                wchar_t tmp [10];
                for (unsigned int i = 0; i<deviceIDInfo->dwPresetIDBytes; i++) {
                    wsprintf(tmp, TEXT("%02X"), arrOutBuff[deviceIDInfo->dwPresetIDOffset+i]);
                    deviceId += tmp;
                }

                /* Convert PlatformID into the string */
                for (unsigned int i = 0; i<deviceIDInfo->dwPlatformIDBytes; i++) {
                    wsprintf(tmp, TEXT("%02X"), arrOutBuff[deviceIDInfo->dwPlatformIDOffset+i]);
                    deviceId += tmp;
                }

                if (deviceId.length() > 30) {
                    deviceId.replace(2, 3,_T(""));
                    deviceId.replace(11, 12,_T(""));
                }
            } else {
                deviceId = TEXT("pocketpc-plugin");
            }
        }
        catch (...) {
            deviceId = TEXT("pocketpc-plugin");
        }
        deviceId.insert(0, TEXT("fwm-"));

        if(deviceId.length() > MAX_DEVID) {
            deviceId.erase(MAX_DEVID);
        }
    }
    else {
        deviceId = TEXT("fwm-pocketpc-plugin");
    }
    return deviceId;
}

#if 0
/*
* Method to get communication settings from device.
* @param name  : name of the property to retrieve value of
* @param value : value of the property
*/

DWORD WINAPI getConfigParameter (wchar_t* name, wchar_t* value) {

    char *tmp = NULL;
    wchar_t* toRet = NULL;
    char  spds_context [128];
    sprintf(spds_context, "%s", SPDS_CONTEXT);
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->readManagementNode(spds_context);

    if (wcscmp(name, SPDM_SYNCURL   ) == 0) {
        tmp = node->readPropertyValue(PROPERTY_SYNC_URL);
    }

    else if (wcscmp(name, SPDM_DEVICEID) == 0) {

        tmp = node->readPropertyValue(PROPERTY_DEVICE_ID);
        if (strcmp(tmp, "") == 0) {
            wstring devId = GetSerialNumberFromKernelIoControl();

            tmp = toMultibyte(devId.c_str());

        }
    }

    else if (wcscmp(name, SPDM_PASSWORD  ) == 0) {
        tmp = node->readPropertyValue(PROPERTY_PASSWORD);
    }
    else if (wcscmp(name, SPDM_USERNAME  ) == 0) {
        tmp = node->readPropertyValue(PROPERTY_USERNAME);
    }
    else if (wcscmp(name, SPDM_SERVERNAME) == 0) {
        tmp = node->readPropertyValue(PROPERTY_SERVER_NAME);
    }
    //else if (wcscmp(name, PROPERTY_SPDM_ENCRYPT) == 0) {
    //    tmp = node->getPropertyValue(PROPERTY_ENCRYPTION);
    //}
    else if (wcscmp(name, PROPERTY_SPDM_MAX_MSG_SIZE) == 0) {
        tmp = node->readPropertyValue(PROPERTY_MAX_MSG_SIZE);
    }
    else if (wcscmp(name, PROPERTY_SPDM_READ_BUFFER_SIZE) == 0) {
        tmp = node->readPropertyValue(PROPERTY_READ_BUFFER_SIZE);
    }

    // overwrite the terminator char \0 of wchar_t* array to read value properly
    toRet = toWideChar(tmp);
    wcscpy(value, toRet);
    value[wcslen(value)] = L' ';

    if (tmp)
        delete [] tmp;
    tmp = NULL;
    if (toRet)
        delete [] toRet;
    toRet = NULL;

    if (node)
        delete node;
    node = NULL;

    return 0;
}


/*
* Method to set communication settings into device. The parameters are retrieved by Visual Basic interface
*
* @param username     : user name
* @param password     : user password
* @param syncURL      : synchronization url
* @param serverName   : the server uri used to address the sync server
* @param errorMsg     : the returned message to vb interface
*/

DWORD WINAPI setConfigParameters(wchar_t *username, wchar_t* password,
                                 wchar_t* syncURL, wchar_t* serverName, wchar_t* errorMsg) {

    wstring deviceId = GetSerialNumberFromKernelIoControl();

    if (wcsstr(syncURL, HTTP_HEADER) == NULL && wcsstr(syncURL, HTTPS_HEADER) == NULL ) {
        wchar_t tmp[256];
        wsprintf(tmp, HTTP_HEADER);
        wcscat(tmp, syncURL);
        wsprintf(syncURL, tmp);
    }

    char  *tmp = NULL;
    char  spds_context [128];
    sprintf(spds_context, "%s", SPDS_CONTEXT);
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->readManagementNode(spds_context);

    node->setPropertyValue(PROPERTY_USERNAME,   _wcc(username));
    node->setPropertyValue(PROPERTY_PASSWORD,   _wcc(password));
    node->setPropertyValue(PROPERTY_SYNC_URL,   _wcc(syncURL));
    node->setPropertyValue(PROPERTY_SERVER_NAME, _wcc(serverName));

    tmp = node->readPropertyValue(PROPERTY_DEVICE_ID);

    if (strcmp(tmp, "") == 0) {
        char *devid = toMultibyte(deviceId.c_str());
        node->setPropertyValue(PROPERTY_DEVICE_ID, devid);
        delete [] devid;
    }

    wsprintf(errorMsg, TEXT("success"));

    // overwrite the terminator char \0 of wchar_t* array to read value properly
    errorMsg[wcslen(errorMsg)] = L' ';

    if (node)
        delete node;
    node = NULL;

    return 0;
}
#endif

/*
NO MORE USED

DWORD WINAPI setConfigParameter(wchar_t* propertyName,
                                  wchar_t* propertyValue, wchar_t* errorMsg) {


    char *tmp = NULL;
    char  spds_context [128];
    sprintf(spds_context, "%s", SPDS_CONTEXT);
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->readManagementNode(spds_context);

    //if (wcscmp(propertyName, PROPERTY_SPDM_ENCRYPT) == 0) {
    //    node->setPropertyValue(PROPERTY_ENCRYPTION, _wcc(propertyValue));  }
    if (wcscmp(propertyName, PROPERTY_SPDM_MAX_MSG_SIZE) == 0) {
        node->setPropertyValue(PROPERTY_MAX_MSG_SIZE, _wcc(propertyValue));
    } else if (wcscmp(propertyName, PROPERTY_SPDM_READ_BUFFER_SIZE) == 0) {
        node->setPropertyValue(PROPERTY_READ_BUFFER_SIZE, _wcc(propertyValue));
    }


    // overwrite the terminator char \0 of wchar_t* array to read value properly
    errorMsg[wcslen(errorMsg)] = L' ';

    if (node)
        delete node;
    node = NULL;

    return 0;
}

*/
/*
* Method to get the synchronization setting parameters.
*
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property


DWORD WINAPI getSourceParameter (LPWSTR sourceName, LPWSTR propertyName,  LPWSTR propertyValue) {

    char *tmp = NULL;
    wchar_t* toRet = NULL;
    char  spds_context [128];

    if (wcscmp(sourceName, SYNC_APPOINTMENTS) == 0) {
        sprintf(spds_context, "%s", APPOINTMENTS_CONTEXT);
    }
    else if (wcscmp(sourceName, SYNC_CONTACTS) == 0) {
        sprintf(spds_context, "%s", CONTACTS_CONTEXT);
    }
    else if (wcscmp(sourceName, SYNC_TASKS) == 0) {
        sprintf(spds_context, "%s", TASKS_CONTEXT);
    }
    else if (wcscmp(sourceName, SYNC_NOTES) == 0) {
        sprintf(spds_context, "%s", NOTES_CONTEXT);
    }
    else if (wcscmp(sourceName, SYNC_FILES) == 0) {
        sprintf(spds_context, "%s", BRIEFCASE_CONTEXT);
    }
    else if (wcscmp(sourceName, SYNC_FAVORITES) == 0) {
        sprintf(spds_context, "%s", FAVORITES_CONTEXT);
    }
    else if (wcscmp(sourceName, SYNC_MAILS) == 0) {
        sprintf(spds_context, "%s", MAILS_CONTEXT);
    }

    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->readManagementNode(spds_context);

    if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SYNC) == 0)  {          // sync
        tmp = node->readPropertyValue(PROPERTY_SOURCE_SYNC);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_LAST) == 0) {
        tmp = node->readPropertyValue(PROPERTY_SOURCE_LAST_SYNC);
    }
     else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_URI) == 0) {    // uri
        tmp = node->readPropertyValue(PROPERTY_SOURCE_URI);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SYNC_MODES) == 0) {   // sync modes
       tmp = node->readPropertyValue(PROPERTY_SOURCE_SYNC_MODES);
    }
    // dir. It is the parameter only for briefcase and notes
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DIR) == 0) {
        tmp = node->readPropertyValue(PROPERTY_SOURCE_DIR);

        char* loc = strchr(tmp, '\\');
        if (loc != NULL) {
        }
        else {
            wchar_t t[128];
            SHGetSpecialFolderPath(NULL, t, CSIDL_PERSONAL, FALSE);
            tmp = toMultibyte(t);
        }
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_INBOX) == 0) {
         tmp = node->readPropertyValue(PROPERTY_SOURCE_INBOX);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_OUTBOX) == 0) {
         tmp = node->readPropertyValue(PROPERTY_SOURCE_OUTBOX);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_TRASH) == 0) {
         tmp = node->readPropertyValue(PROPERTY_SOURCE_TRASH);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SENT) == 0) {
         tmp = node->readPropertyValue(PROPERTY_SOURCE_SENT);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DRAFT) == 0) {
         tmp = node->readPropertyValue(PROPERTY_SOURCE_DRAFT);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_ATTACH_SIZE) == 0) {
        tmp = node->readPropertyValue(PROPERTY_SOURCE_ATTACH_SIZE);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DOWNLOAD_AGE) == 0) {
        tmp = node->readPropertyValue(PROPERTY_SOURCE_DOWNLOAD_AGE);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_BODY_SIZE) == 0) {
        tmp = node->readPropertyValue(PROPERTY_SOURCE_BODY_SIZE);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_USE_SIF) == 0) {
        tmp = node->readPropertyValue(PROPERTY_SOURCE_USE_SIF);
    }

    // overwrite the terminator char \0 of wchar_t* array to read value properly
    toRet = toWideChar(tmp);
    wcscpy(propertyValue, toRet);
    propertyValue[wcslen(propertyValue)] = L' ';

    if (tmp)
        delete [] tmp;
    tmp = NULL;

    if (toRet)
        delete [] toRet;
    toRet = NULL;

    if (node)
        delete node;
    node = NULL;

    return 0;
}

*/

/*
* Method to set the synchronization setting parameters.
*
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property
* @param errorMsg       : returned error message

NO MORE USED

DWORD WINAPI setSourceParameters (wchar_t* sourceName, wchar_t* propertyName,
                                  wchar_t* propertyValue, wchar_t* errorMsg) {

    char  spds_context [128];
    sprintf(spds_context, ROOT_CONTEXT);

    if (wcscmp(sourceName, SYNC_APPOINTMENTS) == 0) {
        sprintf(spds_context, "%s", APPOINTMENTS_CONTEXT);
    }
    else if (wcscmp(sourceName, SYNC_CONTACTS) == 0) {
        sprintf(spds_context, "%s", CONTACTS_CONTEXT);
    }
    else if (wcscmp(sourceName, SYNC_TASKS) == 0) {
        sprintf(spds_context, "%s", TASKS_CONTEXT);
    }
    else if (wcscmp(sourceName, SYNC_NOTES) == 0) {
        sprintf(spds_context, "%s", NOTES_CONTEXT);
    }
    else if (wcscmp(sourceName, SYNC_FILES) == 0) {
        sprintf(spds_context, "%s", BRIEFCASE_CONTEXT);
    }
    else if (wcscmp(sourceName, SYNC_FAVORITES) == 0) {
        sprintf(spds_context, "%s", FAVORITES_CONTEXT);
    }
    else if (wcscmp(sourceName, SYNC_MAILS) == 0) {
        sprintf(spds_context, "%s", MAILS_CONTEXT);
    }

    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->readManagementNode(spds_context);

    if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SYNC) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_SYNC, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_LAST) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_LAST_SYNC, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_URI) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_URI, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DIR) == 0) {

        wchar_t t [256];
        wsprintf(t, propertyValue);

        wchar_t* loc = wcschr(t, L'\\');
        if (loc != NULL)
            wsprintf(t, loc);
        else
            wsprintf(t, TEXT("\\"));

        node->setPropertyValue(PROPERTY_SOURCE_DIR, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_INBOX) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_INBOX, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_OUTBOX) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_OUTBOX, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SENT) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_SENT, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DRAFT) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_DRAFT, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_TRASH) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_TRASH, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_ATTACH_SIZE) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_ATTACH_SIZE, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DOWNLOAD_AGE) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_DOWNLOAD_AGE, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_BODY_SIZE) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_BODY_SIZE, _wcc(propertyValue));
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_USE_SIF) == 0) {
        node->setPropertyValue(PROPERTY_SOURCE_USE_SIF, _wcc(propertyValue));

        //
        // to set the encoding parameter automatically
        //
        if (wcscmp(propertyValue, TEXT("1")) == 0) {
            node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "b64");
        } else {
            node->setPropertyValue(PROPERTY_SOURCE_ENCODING, "bin");
        }

    }

    wsprintf(errorMsg, TEXT("success"));
    errorMsg[wcslen(errorMsg)] = L' ';

    if (node)
        delete node;
    node = NULL;

    return 0;

}
*/
DWORD WINAPI setClientConfiguration(wchar_t* sourceName, wchar_t* propertyName,
                                     wchar_t* propertyValue, wchar_t* errorMsg)  {

    char  spds_context [128];
    sprintf(spds_context, "%s", ROOT_CONTEXT);

    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->readManagementNode(spds_context);

    if (sourceName == NULL || wcscmp(sourceName, TEXT("")) == 0) {

        if (wcscmp(propertyName, PROPERTY_SPDM_LOG_LEVEL) == 0) {
            node->setPropertyValue(PROPERTY_LOG_LEVEL, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_PATH_INFO) == 0) {
            node->setPropertyValue(PROPERTY_PATH_INFO, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_PUSH_NOTIFICATION) == 0) {
            node->setPropertyValue(PROPERTY_PUSH_NOTIFICATION, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_POLLING_NOTIFICATION) == 0) {
            node->setPropertyValue(PROPERTY_POLLING_NOTIFICATION, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_SERVER_NOTIFIED) == 0) {
            node->setPropertyValue(PROPERTY_SERVER_NOTIFIED, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_NOTIFICATION_PORT) == 0) {
            node->setPropertyValue(PROPERTY_NOTIFICATION_PORT, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_SMS_WAP_PUSH) == 0) {
            node->setPropertyValue(PROPERTY_SMS_WAP_PUSH, _wcc(propertyValue));
        } else if (wcscmp(propertyName, PROPERTY_SPDM_PID) == 0) {
            node->setPropertyValue(PROPERTY_PID, _wcc(propertyValue));
        }


    }
    if (node)
        delete node;
    node = NULL;

    return 0;
}

/*
* The method recall the other getClientConfiguration and set the variable isFromDB as true.
* This variable means that the caller is the VB and so it's necessary remove the final char '\0'
* to permit the vb to understand the return string properly
NO MORE USED

DWORD WINAPI getClientConfiguration (wchar_t* sourceName, wchar_t* propertyName,
                                     wchar_t* propertyValue) {

    return getClientConfigurationInternal (sourceName, propertyName, propertyValue, TRUE);
}
*/
/*
* Method to get the client configurations. These property are set by installing process.
* These are extra parameters not used in synchronization process
* but are needed for example to show or hide some fields in GUI interface.
*
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property
*/

DWORD getClientConfigurationInternal (wchar_t* sourceName, wchar_t* propertyName,
                                     wchar_t* propertyValue, BOOL isFromVB) {

    const int VALUESIZE = 1024;
    int nc = 0;

    char  *tmp = NULL;
    wchar_t* toRet = NULL;

    char  spds_context [128];
    sprintf(spds_context, "%s", ROOT_CONTEXT);
    const char arg[] = "dummy";
    DMTree* dmt = DMTreeFactory::getDMTree(arg);
    ManagementNode *node = dmt->readManagementNode(spds_context);

    wchar_t buf[1024];

    /*
    * if sourceName is empty then is a property of root context
    */
    if (sourceName == NULL || wcscmp(sourceName, TEXT("")) == 0) {

        if (wcscmp(propertyName, PROPERTY_SPDM_REMOTE_SETTINGS) == 0) {
            tmp = node->readPropertyValue(PROPERTY_REMOTE_SETTINGS);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_LOG_LEVEL) == 0) {
            tmp = node->readPropertyValue(PROPERTY_LOG_LEVEL);
        }
        //else if (wcscmp(propertyName, PROPERTY_SPDM_ENCRYPT) == 0) {
        //    tmp = node->getPropertyValue(PROPERTY_ENCRYPTION);
        //}
        else if (wcscmp(propertyName, PROPERTY_SPDM_PATH_INFO) == 0) {
            tmp = node->readPropertyValue(PROPERTY_PATH_INFO);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_PUSH_NOTIFICATION) == 0) {
            tmp = node->readPropertyValue(PROPERTY_PUSH_NOTIFICATION);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_NOTIFICATION_PORT) == 0) {
            tmp = node->readPropertyValue(PROPERTY_NOTIFICATION_PORT);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_POLLING_NOTIFICATION) == 0) {
            tmp = node->readPropertyValue(PROPERTY_POLLING_NOTIFICATION);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_SERVER_NOTIFIED) == 0) {
            tmp = node->readPropertyValue(PROPERTY_SERVER_NOTIFIED);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_SMS_WAP_PUSH) == 0) {
            tmp = node->readPropertyValue(PROPERTY_SMS_WAP_PUSH);
        }
        else if (wcscmp(propertyName, PROPERTY_SPDM_PID) == 0) {
            tmp = node->readPropertyValue(PROPERTY_PID);
        }
        else
            tmp = "";

        toRet = toWideChar(tmp);
        wcscpy(propertyValue, toRet);

        if (isFromVB != NULL && isFromVB == TRUE)
            propertyValue[wcslen(propertyValue)] = L' ';

        goto finally;

    }

    wsprintf(buf, TEXT(""));

    wcscpy(propertyValue, buf);
    propertyValue[wcslen(propertyValue)] = L' ';

finally:
    if (toRet)
        delete [] toRet;
    toRet = NULL;

    if(tmp)
        delete [] tmp;
    tmp = NULL;

    if (node) {delete node; node = NULL; }
    return 0;
}




// ***************************************************************************
// Function Name: GetTSPLineDeviceID
//
// Purpose: To get a TSP Line Device ID
//
// Arguments:
//  hLineApp = the HLINEAPP returned by lineInitializeEx
//  dwNumberDevices = also returned by lineInitializeEx
//  dwAPIVersionLow/High = min version of TAPI that we need
//  psTSPLineName = "Cellular Line"
//
// Return Values: Current Device ID
//
// Description:
//  This function returns the device ID of a named TAPI TSP.  The Device ID is
//  used in the call to lineOpen

static DWORD GetTSPLineDeviceID(const HLINEAPP hLineApp,
                         const DWORD dwNumberDevices,
                         const DWORD dwAPIVersionLow,
                         const DWORD dwAPIVersionHigh,
                         const TCHAR* const psTSPLineName)
{
    DWORD dwReturn = 0xffffffff;
    for(DWORD dwCurrentDevID = 0 ; dwCurrentDevID < dwNumberDevices ; dwCurrentDevID++)
    {
        DWORD dwAPIVersion;
        LINEEXTENSIONID LineExtensionID;
        if(0 == lineNegotiateAPIVersion(hLineApp, dwCurrentDevID,
            dwAPIVersionLow, dwAPIVersionHigh,
            &dwAPIVersion, &LineExtensionID))
        {
            LINEDEVCAPS LineDevCaps;
            LineDevCaps.dwTotalSize = sizeof(LineDevCaps);
            if(0 == lineGetDevCaps(hLineApp, dwCurrentDevID,
                dwAPIVersion, 0, &LineDevCaps))
            {
                BYTE* pLineDevCapsBytes = new BYTE[LineDevCaps.dwNeededSize];
                if(0 != pLineDevCapsBytes)
                {
                    LINEDEVCAPS* pLineDevCaps = (LINEDEVCAPS*)pLineDevCapsBytes;
                    pLineDevCaps->dwTotalSize = LineDevCaps.dwNeededSize;
                    if(0 == lineGetDevCaps(hLineApp, dwCurrentDevID,
                        dwAPIVersion, 0, pLineDevCaps))
                    {
                        if(0 == _tcscmp((TCHAR*)((BYTE*)pLineDevCaps+pLineDevCaps->dwLineNameOffset),
                            psTSPLineName))
                        {
                            dwReturn = dwCurrentDevID;
                        }
                    }
                    delete[]  pLineDevCapsBytes;
                }
            }
        }
    }
    return dwReturn;
}

/**
 * Get the phone's IMEI, with the prefix fwm- to be used as deviceID.
 * TODO: make a unique getDeviceId function here with conditional compilation.
 */
wstring getIMEI()
{
    DWORD dwNumDevs;
    DWORD dwAPIVersion = TAPI_API_HIGH_VERSION;
    LINEINITIALIZEEXPARAMS liep;
    HLINEAPP hLineApp = 0;
    HLINE hLine = 0;
    DWORD dwExtVersion;
    LPBYTE pLineGeneralInfoBytes = NULL;
    DWORD dwTAPILineDeviceID;
    const DWORD dwMediaMode = LINEMEDIAMODE_DATAMODEM | LINEMEDIAMODE_INTERACTIVEVOICE;
    LINEGENERALINFO lviGeneralInfo;
    LPLINEGENERALINFO plviGeneralInfo;
    LPTSTR tsSerialNumber;

    wstring ret(TEXT("fwm-smartphone-plugin"));

    // set the line init params
    liep.dwTotalSize = sizeof(liep);
    liep.dwOptions = LINEINITIALIZEEXOPTION_USEEVENT;

    if (lineInitializeEx(&hLineApp, 0, 0,  _T("Funambol"), &dwNumDevs, &dwAPIVersion, &liep))
    {
        goto cleanup;
    }

    // get the device ID
    dwTAPILineDeviceID = GetTSPLineDeviceID(hLineApp, dwNumDevs, TAPI_API_LOW_VERSION, TAPI_API_HIGH_VERSION, CELLTSP_LINENAME_STRING);

    // error getting the line device ID?
    if (0xffffffff == dwTAPILineDeviceID)
    {
        goto cleanup;
    }

    // now try and open the line
    if(lineOpen(hLineApp, dwTAPILineDeviceID,
        &hLine, dwAPIVersion, 0, 0,
        LINECALLPRIVILEGE_OWNER, dwMediaMode, 0))
    {
        goto cleanup;
    }

    // set up ExTAPI
    if (lineNegotiateExtVersion(hLineApp, dwTAPILineDeviceID, dwAPIVersion, EXT_API_LOW_VERSION, EXT_API_HIGH_VERSION, &dwExtVersion))
    {
        goto cleanup;
    }

    // try to get the general info
    lviGeneralInfo.dwTotalSize = sizeof(lviGeneralInfo);

    // step 1: find out how much space we need
    if (lineGetGeneralInfo(hLine, &lviGeneralInfo))
    {
        goto cleanup;
    }

    // step 2: malloc space for all the info we need
    pLineGeneralInfoBytes = new BYTE[lviGeneralInfo.dwNeededSize];
    plviGeneralInfo = (LPLINEGENERALINFO)pLineGeneralInfoBytes;

    // step 3: call lGGI again with the appropriately sized buffer
    if(NULL != pLineGeneralInfoBytes)
    {
        plviGeneralInfo->dwTotalSize = lviGeneralInfo.dwNeededSize;
        if (lineGetGeneralInfo(hLine, plviGeneralInfo))
        {
            goto cleanup;
        }
    } else
    {
        goto cleanup;
    }

    // step 4: cast all the arguments to strings
    if(0 < plviGeneralInfo->dwSerialNumberSize)
    {
        tsSerialNumber = (WCHAR*)(((BYTE*)plviGeneralInfo)+plviGeneralInfo->dwSerialNumberOffset);
        for (unsigned int i = 0; i < wcslen(tsSerialNumber); i++) {

            //
            // for the serial number we force to use only ascii char number and letter
            //

            if(tsSerialNumber[i] >= 32 && tsSerialNumber[i] <= 47)
                tsSerialNumber[i] = (int)tsSerialNumber[i] + 33;
            else if (tsSerialNumber[i] >= 58 && tsSerialNumber[i] <= 64)
                tsSerialNumber[i] = (int)tsSerialNumber[i] + 7;
            else if (tsSerialNumber[i] >= 91 && tsSerialNumber[i] <= 96)
                tsSerialNumber[i] = (int)tsSerialNumber[i] - 6;
            else if (tsSerialNumber[i] >= 123 && tsSerialNumber[i] <= 126)
                tsSerialNumber[i] = (int)tsSerialNumber[i] - 10;
        }
        ret = TEXT("fwm-");
        ret += tsSerialNumber;
        if(ret.length() > MAX_DEVID) {
            ret.erase(MAX_DEVID);
        }
    }

cleanup:
    if (pLineGeneralInfoBytes)
    { delete [] pLineGeneralInfoBytes; pLineGeneralInfoBytes = NULL; }
    if (hLine) lineClose(hLine);
    if (hLineApp) lineShutdown(hLineApp);

    return ret;
}
