/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

#include "pim/mail/MailClient2Server.h"
#include "pim/mail/MailInfo.h"
#include "pim/mail/MailItem.h"
#include "pim/mail/MailClientData.h"
#include "pim/mail/MailSyncSource.h"

#include "pim/account.h"

#include "pim/utils.h"
#include "base/util/utils.h"
#include "base/util/ArrayList.h"
#include <mapidefs.h>
#include <mapicode.h>
#include <mapitags.h>
#include <mapix.h>
#include <mapiutil.h>
#include <cemapi.h>
#include "wtypes.h"

#include "HwndFunctions.h"
#define DEFAULT_USER_W TEXT("default@email")
#define DEFAULT_USER_W_ACUTE TEXT("<default@email>")

//------------------------------------------------------------- Typedef
enum {
    eeePR_ENTRYID,
    eeePR_MESSAGE_FLAGS,
    eeePR_LAST_MODIFICATION_TIME,
    eeeNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eeeNUM_COLS, endSync) =
{
    eeeNUM_COLS,
    PR_ENTRYID,                       // entry id
    PR_MESSAGE_FLAGS,
    PR_LAST_MODIFICATION_TIME        //

};

enum {
    ePR_ENTRYID,
    ePR_MESSAGE_FLAGS,
    //MM
    ePR_LAST_MODIFICATION_TIME,
    ePR_SUBJECT,
    ePR_SUBJECT_PREFIX,
    ePR_MESSAGE_SIZE,
    ePR_SENDER_NAME,
    ePR_SENDER_EMAIL_ADDRESS,
    ePR_HASATTACH,
    ePR_MESSAGE_DELIVERY_TIME,
    ePR_CUSTOM_MESSAGE_ID,
    ePR_CUSTOM_MESSAGE_HEADER,
    ePR_REPLY_RECIPIENT_NAMES,
    ePR_REPLY_RECIPIENT_ENTRIES,
    ePR_CLIENT_SUBMIT_TIME,
    //MM
    NUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(NUM_COLS, properties) =
{
    NUM_COLS,
    PR_ENTRYID,                         // entry id
    PR_MESSAGE_FLAGS,
    //MM
    PR_LAST_MODIFICATION_TIME,          //
    PR_SUBJECT,                         // mail Subject
    PR_SUBJECT_PREFIX,
    PR_MESSAGE_SIZE,                    // sum of the size of every property of the object
    PR_SENDER_NAME,                     // "Marco "
    PR_SENDER_EMAIL_ADDRESS,            // From  "Marco <magi@libero.it>
    PR_HASATTACH,
    PR_MESSAGE_DELIVERY_TIME,
    PR_CUSTOM_MESSAGE_ID,
    PR_CUSTOM_MESSAGE_HEADER,
    PR_REPLY_RECIPIENT_NAMES,
    PR_REPLY_RECIPIENT_ENTRIES,
    PR_CLIENT_SUBMIT_TIME

};

enum {
    eePR_ENTRYID,
    eePR_LAST_MODIFICATION_TIME,
    eePR_DISPLAY_NAME,
    eePR_MESSAGE_FLAGS,
    eeNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eeNUM_COLS, Columns) =
{
    eeNUM_COLS,
    PR_ENTRYID,
    PR_LAST_MODIFICATION_TIME,
    PR_DISPLAY_NAME,
    PR_MESSAGE_FLAGS
};

//--------------------------------------------------------- Local variables

static long mailAll;
static long mailNew;
static long mailUpdated;
static long mailDeleted;
static wchar_t accName[256];
//--------------------------------------------------------- Constructors

MailClient2Server::MailClient2Server() {
    mailAll = -2;
    arrayIndex = 0;
    allTot = 0;
    elementIndex = -2;

    mailNew = -2;
    mailUpdated = -2;
    mailDeleted = -2;
    folderNumber = 0;
    // deletedMessages = new ArrayList();

    entryListArray = NULL;
    folderToSync = NULL;
    newMessagesArray = NULL;
    updatedMessagesArray = NULL;
    deletedMessagesArray = NULL;
    accName[0] = 0;

}

MailClient2Server::~MailClient2Server() {
    //if (mailMessage) {
    //    delete mailMessage; mailMessage = NULL;
    //}
    if (folderToSync) {
        delete [] folderToSync; folderToSync = NULL;
    }
    if (newMessagesArray) {
        for (int i = 0; i < folderNumber; i++)
            //MAPIFreeBuffer(newMessagesArray[i].lpbin);
        delete [] newMessagesArray; newMessagesArray = NULL;
    }
    if (updatedMessagesArray) {
        for (int i = 0; i < folderNumber; i++)
            //MAPIFreeBuffer(updatedMessagesArray[i].lpbin);

        delete [] updatedMessagesArray; updatedMessagesArray = NULL;
    }
    if (deletedMessagesArray) {
        delete [] deletedMessagesArray; deletedMessagesArray = NULL;
    }
    if (entryListArray) {
        for (int i = 0; i < folderNumber; i++)
            //MAPIFreeBuffer(entryListArray[i].lpbin);

        delete [] entryListArray; entryListArray = NULL;
    }
    accName[0] = 0;
}

static void FileTimeToBasicDate(FILETIME ft, BasicTime *b)
{
    SYSTEMTIME t;
    FILETIME loctime;

    FileTimeToLocalFileTime(&ft, &loctime);
    FileTimeToSystemTime(&loctime, &t);

    b->setYear(t.wYear);
    b->setMonth(t.wMonth);
    b->setWeekday(t.wDayOfWeek);
    b->setDay(t.wDay);
    b->setHour(t.wHour);
    b->setMin(t.wMinute);
    b->setSec(t.wSecond);

    time_t tt = FileTimeToUnixTime(ft);

    TIME_ZONE_INFORMATION tzi;
    SafeGetTimeZoneInformation(&tzi);
    int hour = 0;
    double hourDstMinutes = 0;
    int minutes = 0;
    if (IsDST(&tzi, tt)) {
        hour = abs((tzi.Bias + tzi.DaylightBias)/60);
        hourDstMinutes = (tzi.Bias + tzi.DaylightBias)/60;
    }
    else {
        hour = abs(tzi.Bias/60);
        hourDstMinutes = tzi.Bias*1.0/60;
    }
    minutes = abs((int)(hourDstMinutes*60)) - hour*60;
    if (tzi.Bias > 0)
        hour = -hour;
    b->setTzHour(hour);
    b->setTzMin(minutes);
}

/*
* The name of the object has to follow the rule in which the name is
* <FolderName>/<id>
* i.e. I/AAAAABBBCCC   Inbox/AAAAABBBCCC
*
* isUpdate: For update only, the completeObject send all the complete message for Draft folder.
* The others sends only the information extra files
*
* keyOnly: used to get only the key of the item and not the complete. It is used to delete all the items
* before a refresh-from-server
*
*/

MailClientData* MailClient2Server::completeObject(IMessage* pmsg, wchar_t folder, BOOL isUpdate, BOOL keyOnly) {

    SPropValue rgprops[2] = {0};
    ULONG    cProps   = 0;
    ULONG    cmsgValue =  0;
    SPropValue* msgprops = NULL;
    bool read = false;
    char * hdrs = NULL;
    wchar_t *id = NULL;
    MailClientData* mcd = NULL;
    HRESULT hr;

    hr = pmsg->GetProps ((LPSPropTagArray)&properties, MAPI_UNICODE, &cmsgValue,
       &msgprops);
    EXIT_ON_FAILED (hr);

    if ( !msgprops ) {
        LOG.error("completeObject: can't get message props.");
        return NULL;
    }

    if (msgprops[ePR_ENTRYID].ulPropTag != PR_ENTRYID) {
        LOG.error("entry id not found.");
        return NULL;
    }

    id = convertBinaryToWChar(msgprops[ePR_ENTRYID].Value.bin, folder);

    mcd = new MailClientData(id);
    delete [] id;
    id = NULL;

    mcd->setParent(folder);

    if (keyOnly) {
        return mcd;
    }

    //********************* Set EmailData properties ***************************

    // Reset flags (is it still needed?)
    mcd->getEmailData()->setRead(false);
    mcd->getEmailData()->setForwarded(false);
    mcd->getEmailData()->setReplied(false);
    mcd->getEmailData()->setReceived(false);
    mcd->getEmailData()->setCreated(false);
    mcd->getEmailData()->setDeleted(false);
    mcd->getEmailData()->setFlagged(false);

    if (msgprops[ePR_MESSAGE_FLAGS].ulPropTag == PR_MESSAGE_FLAGS) {
        if (msgprops[ePR_MESSAGE_FLAGS].Value.ul & MSGFLAG_READ) {
            read = true;
        }
        else {
            read = false;
        }
    }

    mcd->getEmailData()->setRead(read);

    // If it's an update, and the folder is not draft, do not set the message body
    if (isUpdate && folder != 'D' && folder != 'O'){
        LOG.debug("Update only");
    }
    else {
        /*************************************************************************
         * Past this point, the whole message is sent to the server, and
         * must be complete.
         *************************************************************************/
        FILETIME ft;

        wchar_t time[20];

        if (folder == 'T' && msgprops[ePR_MESSAGE_DELIVERY_TIME].ulPropTag == PR_MESSAGE_DELIVERY_TIME){
            ft = msgprops[ePR_MESSAGE_DELIVERY_TIME].Value.ft;  //OK
        }
        else if (msgprops[ePR_LAST_MODIFICATION_TIME].ulPropTag == PR_LAST_MODIFICATION_TIME){
            ft = msgprops[ePR_LAST_MODIFICATION_TIME].Value.ft;  //OK
        }
        else {
            // Should never happen
            LOG.error("Error retrieving last modification time");
            LOG.error("Using current time");
            SYSTEMTIME st;
            GetLocalTime(&st);
            SystemTimeToFileTime(&st, &ft);
        }
        // Set date in the Email header.
        FileTime2UTCTimeFormatter(ft, time);
        char *t = toMultibyte(time);
        mcd->getEmailData()->setModified(t);
        delete [] t;

        //*************** Set emailItem props *******************************/
        MailMessage& msg = mcd->getEmailData()->getEmailItem();
        ULONG attachCount = 0;

        // Set date ------------------------------------------------------------
        BasicTime bDate;
        
        // In case of Outbox we use always the current time
        if (folder == 'O') {
            SYSTEMTIME systemTime;

            GetSystemTime(&systemTime);
            SystemTimeToFileTime(&systemTime, &ft);
        }
       
        FileTimeToBasicDate(ft, &bDate);
        msg.setDate(bDate);

        // Set subject ------------------------------------------------------------
        if (msgprops[ePR_SUBJECT].ulPropTag == PR_SUBJECT) {
            msg.setSubject(_wcc(msgprops[ePR_SUBJECT].Value.lpszW));
        }

        wchar_t* emailSender = NULL;
        wchar_t* sender      = NULL;
        wchar_t* accountName = NULL;
        wchar_t*  fr         = NULL;
        // Set sender address -----------------------------------------------------
        if (msgprops[ePR_SENDER_EMAIL_ADDRESS].ulPropTag == PR_SENDER_EMAIL_ADDRESS) {

            LOG.debug("MailClientToServer: PR_SENDER_EMAIL_ADDRESS: %ls", msgprops[ePR_SENDER_EMAIL_ADDRESS].Value.lpszW);

            emailSender = msgprops[ePR_SENDER_EMAIL_ADDRESS].Value.lpszW;

            if (msgprops[ePR_SENDER_NAME].ulPropTag == PR_SENDER_NAME) {
                sender = msgprops[ePR_SENDER_NAME].Value.lpszW;
            }

            if (sender != NULL && wcscmp(sender, emailSender) == 0 && folder == 'O') {

                    //wchar_t accountMail[256];
                    //accountName = new wchar_t[256];
                    //getAccountInfo(accountName, accountMail);
                    if (wcscmp(emailSender, DEFAULT_USER_W) == 0) {
                        msg.setFrom("");
                    } else {
                        fr = new wchar_t[wcslen(accName) + wcslen(emailSender) + 10];
                        wsprintf(fr, TEXT("\"%s\" <%s>"), accName, emailSender );
                        msg.setFrom(_wcc(fr));
                        delete [] fr; fr = NULL;
                    }

            } else {
                // if the email is default user it means the sender is empty and the user
                // want to use the default mail added by the server.
                // The check wcsstr of the email sender is on the acute only to avoid
                // something like default@email.com that can be valid
                if (wcscmp(emailSender, DEFAULT_USER_W) == 0 ||
                    wcsstr(emailSender, DEFAULT_USER_W_ACUTE) != 0) {
                    msg.setFrom("");
                } else {
                    msg.setFrom(_wcc(emailSender));
                }



            }
            //msg.setFrom(_wcc(emailSender));
            LOG.debug("MailClientToServer: From: %s\n", msg.getFrom());
        }

        // Set recipients ---------------------------------------------------------
        char* to = NULL, *cc = NULL, *bcc = NULL;
        hr = GetRecipients(pmsg, &to, &cc, &bcc);
        EXIT_ON_FAILED (hr);

        msg.setTo (to);
        msg.setCc (cc);
        msg.setBcc(bcc);

        if (to) delete [] to;
        if (cc) delete [] cc;
        if (bcc) delete [] bcc;

        // Set body -----------------------------------------------------------
        BodyPart bodyPart;
        wchar_t *buffer = NULL;
        char* body = NULL;
        hr = GetBody(pmsg, &buffer);
        EXIT_ON_FAILED (hr);

        body = toMultibyte(buffer);
        delete [] buffer;

        if (!body) {
            LOG.error("null body for message %s", msg.getSubject());
            delete mcd; mcd = 0;
            goto FuncExit;
        }

        bodyPart.setContent(body);
        msg.setBody(bodyPart);
        delete [] body;

        // Set attachments ----------------------------------------------------
        if (msgprops[ePR_HASATTACH].ulPropTag == PR_HASATTACH) {
            attachCount = msgprops[ePR_HASATTACH].Value.ul;
            GetAttachments(pmsg, mcd);
        }

        // Message Id ---------------------------------------------------------
        char *msgId = NULL;
        if (msgprops[ePR_CUSTOM_MESSAGE_ID].ulPropTag == PR_CUSTOM_MESSAGE_ID) {
            msgId = toMultibyte(msgprops[ePR_CUSTOM_MESSAGE_ID].Value.lpszW);
            if (!msgId) {
                LOG.error("completeObject: error retrieving messageId from POOM");
            }
        }
        else {
            msgId = createMessageID(mcd->getEntryId());
            id = toWideChar(msgId);
            if (id) {
                // Set message id in POOM
                rgprops[cProps].ulPropTag = PR_CUSTOM_MESSAGE_ID;
                rgprops[cProps].Value.lpszW  = id;
                ++cProps;

                rgprops[cProps].ulPropTag = PR_LAST_MODIFICATION_TIME;
                rgprops[cProps].Value.ft  = ft;
                ++cProps;

            }
            else {
                LOG.error("completeObject: error converting message id: %s", id);
            }

        }
        // Set message id in outgoing message
        msg.setMessageId(msgId);
        delete [] msgId;

        // Other headers --------------------------------------------------------
        if (msgprops[ePR_CUSTOM_MESSAGE_HEADER].ulPropTag ==
            PR_CUSTOM_MESSAGE_HEADER) {
            hdrs = toMultibyte(msgprops[ePR_CUSTOM_MESSAGE_HEADER].Value.lpszW);
            if (hdrs) {
                msg.setHeaders(hdrs);
                //delete [] hdrs;
            }
            else {
                LOG.error("completeObject: error retrieving headers.");
            }
        }

        if (cProps > 0) {
            HRESULT hhr = pmsg->SetProps(cProps, rgprops, NULL);
            int k = 0;
        }
    }

FuncExit:
    if (id) { delete [] id; id = NULL; }
    if (hdrs) {delete [] hdrs; hdrs = NULL; }
    return mcd;
}

IMAPISession* MailClient2Server::logOn() {

    IMAPISession *    pSession    =  NULL;
    HRESULT hr;
    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);

    if (hr == S_OK)
        return pSession;
    else
        return NULL;

}

void MailClient2Server::logOff(IMAPISession* pSession) {
    if (pSession)
        RELEASE_OBJ (pSession);
}

MailClientData* MailClient2Server::getFirstItem(BOOL keyOnly) {
    LOG.debug("Enter into getFirstItem function");
    allTot = 0;
    mailAll = -2;
    elementIndex = -2;

    for (int i = 0; i < folderNumber; i++) {
        allTot += entryListArray[i].cValues;
    }

    // send nr of mails from client, slow-sync, set -1 to wParam to signal that these are items from client
    if(HwndFunctions::wnd)
        SendMessage(HwndFunctions::wnd, ID_MYMSG_TOTAL_ITEMS, SOURCE_MAIL, (LPARAM)allTot);

    if (allTot == 0) {
        LOG.debug("Exiting from getFirstItem function. No more item");
        return NULL;
    }

    if (allTot > 0) {
        mailAll++;       // from -2 -> -1
        elementIndex++;  // from -2 -> -1
    }
    arrayIndex = 0;
    LOG.debug("Exiting from getFirstItem function");
    return getNextItem(keyOnly);

}

MailClientData* MailClient2Server::getNextItem(BOOL keyOnly) {

    LOG.debug("Enter into getNextItem function");

    IMAPISession *    pSession    = NULL;
    HRESULT           hr          = NULL;
    IMessage*         pmsg        = NULL;
    ULONG             returned    = 0;
    int               tot         = 0;

    if (allTot == 0) {
        LOG.debug("Exiting from getNextItem function. No more item");
        return NULL;
    }
    MailClientData* mailClientData = NULL;

    mailAll++;
    elementIndex++;
    if (arrayIndex == folderNumber)
        return NULL;
      // if the current folder has no updated items, increments arrayIndex immediately

    if (entryListArray[arrayIndex].cValues == 0) {
        for (int k = 0; (k < folderNumber && arrayIndex < folderNumber); k++) {
            if (entryListArray[arrayIndex].cValues == 0)
                arrayIndex++;
             else
                break;
        }
    }
    pSession = logOn();

    if (pSession == NULL)
        goto FuncExit;

    if (mailAll < allTot) {

        hr = pSession->OpenEntry(entryListArray[arrayIndex].lpbin[elementIndex].cb,
            (ENTRYID*)entryListArray[arrayIndex].lpbin[elementIndex].lpb,
            NULL,
            MAPI_BEST_ACCESS,
            &returned,
            (IUnknown **) &pmsg);
        EXIT_ON_FAILED (hr);
        ASSERT (pmsg);

        mailClientData = completeObject(pmsg, folderToSync[arrayIndex], FALSE, keyOnly);

        if ((unsigned long)elementIndex == entryListArray[arrayIndex].cValues -1) {
            arrayIndex++;
            elementIndex = -1;
        }

    }

    logOff(pSession);

FuncExit:
    LOG.debug("Exiting from getNextItem function");
    return mailClientData;
}

MailClientData* MailClient2Server::getFirstNewItem() {

    LOG.debug("Enter into getFirstNewItem function");
    allTot = 0;
    mailNew = -2;       // from -2 -> -1
    elementIndex = -2;  // from -2 -> -1

    for (int i = 0; i < folderNumber; i++) {
        allTot += newMessagesArray[i].cValues;
    }

    if (allTot == 0) {
        LOG.debug("Exiting from getFirstNewItem function. No more item");
        return NULL;
    }

    if (allTot > 0) {
        mailNew++;       // from -2 -> -1
        elementIndex++;  // from -2 -> -1
    }
    arrayIndex = 0;

    LOG.debug("Exiting from getFirstNewItem function");
    return getNextNewItem();

}

MailClientData* MailClient2Server::getNextNewItem() {

    LOG.debug("Enter into getNextNewItem function");
    IMAPISession *    pSession    = NULL;
    HRESULT           hr          = NULL;
    IMessage*         pmsg        = NULL;
    ULONG             returned    = 0;
    MailClientData*   mailClientData = NULL;

    if (allTot == 0) {
        LOG.debug("Exiting from getNextNewItem function. No more item");
        return NULL;
    }

    mailNew++;
    elementIndex++;

     if (arrayIndex == folderNumber)
        return NULL;
     // if the current folder has no updated items, increments arrayIndex immediately
    if (newMessagesArray[arrayIndex].cValues == 0) {
        for (int k = 0; (k < folderNumber && arrayIndex < folderNumber); k++) {
            if (newMessagesArray[arrayIndex].cValues == 0)
                arrayIndex++;
            else
                break;
        }
    }

    pSession = logOn();

    if (pSession == NULL)
        goto FuncExit;

    if (mailNew < allTot) {

        hr = pSession->OpenEntry(newMessagesArray[arrayIndex].lpbin[elementIndex].cb,
            (ENTRYID*)newMessagesArray[arrayIndex].lpbin[elementIndex].lpb,
            NULL,
            MAPI_BEST_ACCESS,
            &returned,
            (IUnknown **) &pmsg);
        EXIT_ON_FAILED (hr);
        ASSERT (pmsg);

        mailClientData = completeObject(pmsg, folderToSync[arrayIndex], FALSE);

        if ((unsigned long)elementIndex == newMessagesArray[arrayIndex].cValues -1) {
            arrayIndex++;
            elementIndex = -1;
        }
    }

    logOff(pSession);

FuncExit:
    LOG.debug("Exiting from getNextNewItem function");
    return mailClientData;
}


MailClientData* MailClient2Server::getFirstUpdatedItem() {

    LOG.debug("Enter into getFirstUpdatedItem function");
    allTot = 0;
    mailUpdated = -2;       // from -2 -> -1
    elementIndex = -2;  // from -2 -> -1

    for (int i = 0; i < folderNumber; i++) {
        allTot += updatedMessagesArray[i].cValues;
    }

    if (allTot == 0) {
        LOG.debug("Exiting from getFirstUpdatedItem function. No more item");
        return NULL;
    }

    if (allTot > 0) {
        mailUpdated++;       // from -2 -> -1
        elementIndex++;  // from -2 -> -1
    }
    arrayIndex = 0;

    LOG.debug("Exiting from getFirstUpdatedItem function");
    return getNextUpdatedItem();

}

MailClientData* MailClient2Server::getNextUpdatedItem() {

    LOG.debug("Enter into getNextUpdatedItem function");
    IMAPISession *    pSession    = NULL;
    HRESULT           hr          = NULL;
    IMessage*         pmsg        = NULL;
    ULONG             returned    = 0;
    MailClientData*   mailClientData = NULL;

    if (allTot == 0) {
        LOG.debug("Exiting from getNextUpdatedItem function. No more item");
        return NULL;
    }

    mailUpdated++;
    elementIndex++;

     if (arrayIndex == folderNumber)
        return NULL;
    // if the current folder has no updated items, increments arrayIndex immediately
    if (updatedMessagesArray[arrayIndex].cValues == 0) {
        for (int k = 0; (k < folderNumber && arrayIndex < folderNumber); k++) {
            if (updatedMessagesArray[arrayIndex].cValues == 0)
                arrayIndex++;
            else
                break;
        }
    }
    pSession = logOn();

    if (pSession == NULL)
        goto FuncExit;

    if (mailUpdated < allTot) {

        hr = pSession->OpenEntry(updatedMessagesArray[arrayIndex].lpbin[elementIndex].cb,
            (ENTRYID*)updatedMessagesArray[arrayIndex].lpbin[elementIndex].lpb,
            NULL,
            MAPI_BEST_ACCESS,
            &returned,
            (IUnknown **) &pmsg);
        EXIT_ON_FAILED (hr);
        ASSERT (pmsg);

        //
        // The updated parameter
        //
        mailClientData = completeObject(pmsg, folderToSync[arrayIndex], TRUE);

        if ((unsigned long)elementIndex == updatedMessagesArray[arrayIndex].cValues -1) {
            arrayIndex++;
            elementIndex = -1;
        }
    }

    logOff(pSession);

FuncExit:
    LOG.debug("Exiting from getNextUpdatedItem function");
    return mailClientData;
}

MailClientData* MailClient2Server::getFirstDeletedItem() {

    LOG.debug("Enter into getFirstDeletedItem function");
    allTot = 0;
    mailDeleted  = -2;  // from -2 -> -1
    elementIndex = -2;  // from -2 -> -1

    for (int i = 0; i < folderNumber; i++) {
        allTot += deletedMessagesArray[i]->size();
    }

    if (allTot == 0) {
        LOG.debug("Exiting from getFirstDeletedItem function. No more item");
        return NULL;
    }

    if (allTot > 0) {
        mailDeleted++;       // from -2 -> -1
        elementIndex++;  // from -2 -> -1
    }
    arrayIndex = 0;

    LOG.debug("Exiting from getFirstDeletedItem function");
    return getNextDeletedItem();

}

MailClientData* MailClient2Server::getNextDeletedItem() {

    LOG.debug("Enter into getNextDeletedItem function");

    if (allTot == 0) {
        LOG.debug("Exiting from getNextDeletedItem function. No more item");
        return NULL;
    }

    MailClientData* mailClientData = NULL;
    wchar_t* r = NULL;
    mailDeleted++;
    elementIndex++;

    // if arrayIndex is the same as folder number we are at the end of the array
    if (arrayIndex == folderNumber)
        return NULL;

    // if the current folder has no deleted items, increments arrayIndex immediately
    if (deletedMessagesArray[arrayIndex]->size() == 0) {
        for (int k = 0; (k < folderNumber && arrayIndex < folderNumber); k++) {
            if (deletedMessagesArray[arrayIndex]->size() == 0)
                arrayIndex++;
            else
                break;
        }
    }

    if (mailDeleted < allTot) {

        MailItem* mailItem = (MailItem*)deletedMessagesArray[arrayIndex]->get(elementIndex);

        if (mailItem) {
            wchar_t *id = toWideChar(mailItem->getId());
            if (!id){
                LOG.error("Can't convert entryId");
                return NULL;
            }
            mailClientData = new MailClientData(id);
            delete [] id;
            mailClientData->setParent(folderToSync[arrayIndex]);
        }

        if (elementIndex == deletedMessagesArray[arrayIndex]->size() -1) {
            arrayIndex++;
            elementIndex = -1;
        }
    }
    LOG.debug("Exiting from getNextDeletedItem function");
    return mailClientData;
}

BOOL openTmpFile(wchar_t* filename, FILE** f) {

    BOOL ret = TRUE;
    *f = _wfopen(filename, TEXT("w+b"));

    if(f == NULL)
        ret = FALSE;

    return ret;
}

void closeTmpFile(FILE* f) {

    if (f) {
        fflush(f);
        fclose(f);
    }
}

int writeToTmpFile(FILE* f, char *ptr, int numBytes) {

   int numWritten = fwrite(ptr, sizeof(char), numBytes, f);
   return numWritten;

}

HRESULT MailClient2Server::GetAttachments(IMessage* pmsg, MailClientData* mcd) {

    LOG.debug("Enter into GetAttachments function");
    LPMAPITABLE pAttachTable = NULL;
    // for each row in the attach table
    LPSRowSet pRows = NULL;
    int currentRow = 0;
    char szBuf[2048];
    ULONG ulNumChars;

    HRESULT hr = pmsg->GetAttachmentTable(0, &pAttachTable);
    EXIT_ON_FAILED(hr);

    // iterate all the attachment
    while(TRUE) {

        hr = pAttachTable->QueryRows(1, 0, &pRows);
        if (FAILED(hr) || pRows->cRows != 1)
            break;
        currentRow++;

        // only one row is returned anyway
        int nProps = pRows->aRow[0].cValues;
        long num = -1;
        long size = -1;
        wchar_t* filename = NULL, *filenameTmp = NULL;
        long msgstatus = -1;
        IAttach* attach;

        for (int i=0; i<nProps; i++) {
            ULONG ulPT = pRows->aRow[0].lpProps[i].ulPropTag;
            // loop through all the properties

            // TCHAR* szPropName = ReverseLookup(ulPT);
            switch (ulPT) {
                case PR_ATTACH_NUM:
                    num = pRows->aRow[0].lpProps[i].Value.ul;
                    break;
                case PR_ATTACH_SIZE:
                    size = pRows->aRow[0].lpProps[i].Value.ul;
                    break;
                case PR_ATTACH_FILENAME:
                    filenameTmp = pRows->aRow[0].lpProps[i].Value.lpszW;
                    if (filenameTmp)
                        filename = wstrdup(filenameTmp);
                    break;
                case PR_MSG_STATUS:
                    msgstatus = pRows->aRow[0].lpProps[i].Value.ul;
                    break;
                case 0x30000003:
                    // pRows->aRow[0].lpProps[ii].Value.ul: sequence number
                    break;
                default:
                    break;
            }    // switch

        }
        FreeProws(pRows);

        pmsg->OpenAttach(num, NULL, 0, &attach);
        // read IStream->Read way to write into an array of char.
        IStream* pStream;
        hr = attach->OpenProperty(PR_ATTACH_DATA_BIN, 0, NULL, ATTACH_BY_VALUE, (LPUNKNOWN *)&pStream);
        EXIT_ON_FAILED(hr);

        wchar_t tmpPath[64];
        wchar_t tmpFileName[MAX_PATH];  // System constants for the path

        GetTempPath(64, tmpPath);
        GetTempFileName(tmpPath, TEXT("fun"), 0, tmpFileName);

        FILE* f = NULL;
        f = _wfopen(tmpFileName, TEXT("w+b"));

        if (f == NULL)
            goto FuncExit;

        int m = 0;
        do {
            pStream->Read(szBuf, 2048, &ulNumChars);
            writeToTmpFile(f, szBuf, ulNumChars);
            m = m + ulNumChars;

        } while (ulNumChars >= 2048);

        closeTmpFile(f);

        pStream->Release();

        BodyPart bodyPart;
        bodyPart.setFilename(_wcc(filename));
        bodyPart.setContent(_wcc(tmpFileName));
        bodyPart.setEncoding("base64"); // currently the ppc default

        if (wcsstr(filename, TEXT(".wav"))) {
            bodyPart.setMimeType("audio/x-wav");
        } else if (wcsstr(filename, TEXT(".bmp"))) {
            bodyPart.setMimeType("image/x-ms-bmp");
        } else if (wcsstr(filename, TEXT(".gif"))) {
            bodyPart.setMimeType("image/gif");
        } else if (wcsstr(filename, TEXT(".jpg"))) {
            bodyPart.setMimeType("image/jpg");
        } else if (wcsstr(filename, TEXT(".psw"))) {
            bodyPart.setMimeType("application/msword");
        } else if (wcsstr(filename, TEXT(".htm"))) {
            bodyPart.setMimeType("text/html");
        }

        mcd->getEmailData()->getEmailItem().addAttachment(bodyPart);

        if (filename)
            delete [] filename;

    }
FuncExit:
    // Free each element in the pRows buffer
    FreeProws(pRows);
    RELEASE_OBJ(pAttachTable);
    LOG.debug("Exiting from GetAttachments function");
    return hr;

}


HRESULT MailClient2Server::GetBody(IMessage* pmsg, wchar_t** body) {

    LOG.debug("Enter into GetBody function");
    IStream* pStream;
    ULONG ulNumChars;
    ULONG ulPos = 0;
    HRESULT    hr;
    ULARGE_INTEGER plibNewPosition, startPosition;
    LARGE_INTEGER dlibMove;
    dlibMove.QuadPart= 0;

    // Open the Stream to the body
    hr = pmsg->OpenProperty(PR_BODY, NULL, STGM_READ, NULL,(LPUNKNOWN *) &pStream);
    EXIT_ON_FAILED(hr);
    pStream->Seek( dlibMove, STREAM_SEEK_END, &plibNewPosition);

    *body = new wchar_t[(size_t)plibNewPosition.QuadPart + 1];
    memset(*body, 0, ((size_t)plibNewPosition.QuadPart + 1)* sizeof(wchar_t));

    pStream->Seek( dlibMove, STREAM_SEEK_SET, &startPosition);

    hr = pStream->Read(*body, (size_t)plibNewPosition.QuadPart, &ulNumChars);
    EXIT_ON_FAILED(hr);

    pStream->Release();

    return S_OK;


FuncExit:
    LOG.error("Error reading body mail");
    return -1;
}

wstring BuildAddress(IN LPTSTR pszDisplayName, IN LPTSTR pszEmailAddress)
{
    LOG.debug("Enter into BuildAddress function");
    wstring addressBuilt;
    if (pszDisplayName == NULL && pszEmailAddress !=NULL ) {
        addressBuilt.append(pszEmailAddress);
    } else if (pszDisplayName!=NULL && pszEmailAddress==NULL) {
        addressBuilt.append(TEXT("\""));
        addressBuilt.append(pszDisplayName);
        addressBuilt.append(TEXT("\""));
    } else if (pszDisplayName!=NULL && pszEmailAddress!=NULL) {
        addressBuilt.append(TEXT("\""));
        addressBuilt.append(pszDisplayName);
        addressBuilt.append(TEXT("\""));
        addressBuilt.append(TEXT(" <"));
        addressBuilt.append(pszEmailAddress);
        addressBuilt.append(TEXT(">"));
    } else {
        addressBuilt.append(TEXT("UNKNOWN"));
    }
    LOG.debug("Exiting from BuildAddress function");
    return addressBuilt;
}

// concatenate the recipients part
HRESULT MailClient2Server::GetRecipients(LPMESSAGE pMessage, char** to, char** cc, char** bcc) {

    LOG.debug("Enter into GetRecipients function");
    // for the recipient table
    LPMAPITABLE pRecipientTable = NULL;
    // for each row in the recipient table
    LPSRowSet pRows = NULL;
    int currentRow  = 0;
    int nTempLen    = 0;
    wstring::iterator it;

    wstring szTempTO, szTempCC, szTempBCC;

    // pRecipientTable->SetColumn does not work. Returning result is sth. like "NOT IMPLEMENTED".
    // loop through all rows without limiting the columns
    HRESULT hr = pMessage->GetRecipientTable(0, &pRecipientTable);
    EXIT_ON_FAILED(hr);

    // iterate all the recipients
    while(TRUE) {

        hr = pRecipientTable->QueryRows(1, 0, &pRows);
        if (FAILED(hr) || pRows->cRows != 1) break;
        currentRow++;

        // only one row is returned anyway
        int nProps = pRows->aRow[0].cValues;

        TCHAR *szReceipName, *szReceipEmail;
        ULONG ulReceipType;
        szReceipEmail = NULL;
        szReceipName = NULL;
        ulReceipType = 0;

        // loop through all the properties
        for (int ii=0; ii<nProps; ii++) {
            ULONG ulPT = pRows->aRow[0].lpProps[ii].ulPropTag;

            // TCHAR* szPropName = ReverseLookup(ulPT);
            switch (ulPT) {
            case PR_EMAIL_ADDRESS:
                szReceipEmail = pRows->aRow[0].lpProps[ii].Value.LPSZ;
                break;
            case PR_RECIPIENT_TYPE:
                ulReceipType = pRows->aRow[0].lpProps[ii].Value.ul;
                break;
            case PR_DISPLAY_NAME:
                szReceipName = pRows->aRow[0].lpProps[ii].Value.LPSZ;
                break;
            case PR_ADDRTYPE:
                // pRows->aRow[0].lpProps[ii].Value.LPSZ. Always "SMTP"
                break;
            case 0x30000003:
                // pRows->aRow[0].lpProps[ii].Value.ul: sequence number
                break;
            default:
                break;
            }    // switch
        }

        // now let us build the String
        wstring szAddr = BuildAddress(szReceipName, szReceipEmail);
        if (ulReceipType==MAPI_TO) {
            szTempTO.append(szAddr);
            szTempTO.append(TEXT(","));
        } else if (ulReceipType==MAPI_CC) {
            szTempCC.append(szAddr);
            szTempCC.append(TEXT(","));
        } else if (ulReceipType==MAPI_BCC) {
            szTempBCC.append(szAddr);
            szTempBCC.append(TEXT(","));
        }

        // Free each element in the pRows buffer
        FreeProws(pRows);
    }    // end of the loop through all recipients

    // remove the trailing ",", if any
    int nLen = 0;

    nLen = szTempTO.size();
    if (nLen > 0) {
        it = szTempTO.end() -1;
        if (*it == L',')
            *it = L'\0';
    }

    nLen = szTempCC.size();
    if (nLen > 0) {
        it = szTempCC.end() -1;
        if (*it == L',')
            *it = L'\0';
    }

    nLen = szTempBCC.size();
    if (nLen > 0) {
        it = szTempBCC.end() -1;
        if (*it == L',')
            *it = L'\0';
    }

    nTempLen = szTempTO.size();
    if (nTempLen>0) {
        *to = toMultibyte(szTempTO.c_str());
    }
    nTempLen = szTempCC.size();
    if (nTempLen>0) {
        *cc = toMultibyte(szTempCC.c_str());
    }
    nTempLen = szTempBCC.size();
    if (nTempLen>0) {
        *bcc = toMultibyte(szTempBCC.c_str());
    }

FuncExit:
    // Free each element in the pRows buffer
    FreeProws(pRows);
    RELEASE_OBJ(pRecipientTable);
    LOG.debug("Exiting from GetRecipients function");
    return hr;
}

HRESULT MailClient2Server::GetAllMessages(ENTRYLIST& entryList, LPMAPIFOLDER m_pInBoxFolder,
                                          const wchar_t* path, wchar_t folderId) {
    LOG.debug("Enter into GetAllMessages function");
    HRESULT        hr;
    UINT        nEntries = 0;
    SBinary        sbEntry;
    LPMAPITABLE pContentsTable = NULL;
    LPSRowSet    pRows = NULL;

    // We define a SPropTagArray array here using the SizedSPropTagArray Macro
    // This enum will allows us to access portions of the array by a name instead of a number.
    // If more tags are added to the array, appropriate constants need to be added to the enum.

    hr = m_pInBoxFolder->GetContentsTable( 0, &pContentsTable);
    EXIT_ON_FAILED(hr);

    hr = pContentsTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);

    while(!FAILED(hr = pContentsTable->QueryRows(1, 0, &pRows)))
    {
        if(pRows->cRows == 0)
            break;

        sbEntry = pRows->aRow[0].lpProps[eePR_ENTRYID].Value.bin;

        if(entryList.cValues)
        {
            PVOID    pOldBuffer = entryList.lpbin;
            MAPIAllocateBuffer(sizeof(SBinary) * (entryList.cValues + 1), (LPVOID*)&(entryList.lpbin));
            memmove((LPVOID*)entryList.lpbin, pOldBuffer, sizeof(SBinary) * entryList.cValues);
            MAPIFreeBuffer(pOldBuffer);
        }
        else {
            MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(entryList.lpbin));
        }

        entryList.lpbin[entryList.cValues].cb = sbEntry.cb;
        MAPIAllocateBuffer(sbEntry.cb, (LPVOID*)&(entryList.lpbin[entryList.cValues].lpb) );
        memmove(entryList.lpbin[entryList.cValues].lpb, sbEntry.lpb, sbEntry.cb);
        entryList.cValues++;

        // Free each element in the pRows buffer
        FreeProws(pRows);

        hr = S_OK;
    }

FuncExit:
    if(pRows)
    {
        // Free each element in the pRows buffer
        FreeProws(pRows);
    }
    RELEASE_OBJ(pContentsTable);
    LOG.debug("Exiting from GetAllMessages function");
    return hr;
}


HRESULT MailClient2Server::GetModificatedMessages(
                                ENTRYLIST& newMessages,
                                ENTRYLIST& updatedMessages,
                                ArrayList** deletedMessages,
                                LPMAPIFOLDER m_pInBoxFolder,
                                const wchar_t* path,
                                wchar_t folderId) {

    LOG.debug("Enter into GetModificatedMessages function");
    HRESULT        hr;
    UINT        nEntries = 0;
    SBinary        sbEntry;
    LPMAPITABLE pContentsTable = NULL;
    LPSRowSet    pRows = NULL;
    DATE        lastSync = 0;
    int         numMailItem = 0;
    int         i = 0;
    int         action = 0;
    BOOL        found = FALSE;

    BOOL        flagRead = FALSE;
    FILETIME    ft;
    SYSTEMTIME  st;

    DATE        lastSyncMailItem = 0;
    BOOL        flagReadMailItem = FALSE;

    MailInfo*  mailInfo = readPreviousMail(path, folderId);

    if (mailInfo == NULL) {
        if (folderId == 'O') {
            mailInfo = new MailInfo();
        } else {
            goto FuncExit;
        }
    }

    MailItem*  mailItem = NULL;

    lastSync    = mailInfo->getLast();
    numMailItem = mailInfo->getNum();


    // We define a SPropTagArray array here using the SizedSPropTagArray Macro
    // This enum will allows us to access portions of the array by a name instead of a number.
    // If more tags are added to the array, appropriate constants need to be added to the enum.

    hr = m_pInBoxFolder->GetContentsTable( 0, &pContentsTable);
    EXIT_ON_FAILED(hr);

    hr = pContentsTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);

    while(!FAILED(hr = pContentsTable->QueryRows(1, 0, &pRows)))
    {
        found = FALSE;

        if(pRows->cRows == 0)
            break;

        // get the property of the mail in the folder
        sbEntry = pRows->aRow[0].lpProps[eePR_ENTRYID].Value.bin;
        ft = pRows->aRow[0].lpProps[eePR_LAST_MODIFICATION_TIME].Value.ft;
        FileTimeToSystemTime(&ft, &st);
        SystemTimeToVariantTime(&st, &lastSyncMailItem);

        if (pRows->aRow[0].lpProps[eePR_MESSAGE_FLAGS].Value.ul & MSGFLAG_READ)
            flagRead = TRUE;
        else
            flagRead = FALSE;

        char* id = convertBinaryToChar(sbEntry, folderId);

        for (i = 0; i < mailInfo->getMailItems()->size(); i++) {

            mailItem = (MailItem*)(mailInfo->getMailItems())->get(i);;

            if (strcmp(id, mailItem->getId()) != 0)
                continue;

            // if no continue then it goes on and found the item
            found = TRUE;
            flagReadMailItem = mailItem->getRead();

            if (flagReadMailItem != flagRead ||
                lastSyncMailItem > lastSync) {
                action = UPDATED_MAIL;
            } else {
                action = NO_ACTION_MAIL;
            }
            mailInfo->getMailItems()->removeElementAt(i);
            break;

        }

        if (id) { delete [] id; id = NULL;}

        if (found == FALSE)
            action = NEW_MAIL;

        switch (action) {

            case NEW_MAIL: {

                if(newMessages.cValues)
                {
                    PVOID    pOldBuffer = newMessages.lpbin;
                    MAPIAllocateBuffer(sizeof(SBinary) * (newMessages.cValues + 1), (LPVOID*)&(newMessages.lpbin));
                    memmove((LPVOID*)newMessages.lpbin, pOldBuffer, sizeof(SBinary) * newMessages.cValues);
                    MAPIFreeBuffer(pOldBuffer);
                }
                else {
                    MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(newMessages.lpbin));
                }
                newMessages.lpbin[newMessages.cValues].cb = sbEntry.cb;
                MAPIAllocateBuffer(sbEntry.cb, (LPVOID*)&(newMessages.lpbin[newMessages.cValues].lpb) );
                memmove(newMessages.lpbin[newMessages.cValues].lpb, sbEntry.lpb, sbEntry.cb);
                newMessages.cValues++;

                // Free each element in the pRows buffer
                FreeProws(pRows);

            }; break;

            case UPDATED_MAIL: {

                if(updatedMessages.cValues)
                {
                    PVOID    pOldBuffer = updatedMessages.lpbin;
                    MAPIAllocateBuffer(sizeof(SBinary) * (updatedMessages.cValues + 1), (LPVOID*)&(updatedMessages.lpbin));
                    memmove((LPVOID*)updatedMessages.lpbin, pOldBuffer, sizeof(SBinary) * updatedMessages.cValues);
                    MAPIFreeBuffer(pOldBuffer);
                }
                else {
                    MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(updatedMessages.lpbin));
                }

                updatedMessages.lpbin[updatedMessages.cValues].cb = sbEntry.cb;
                MAPIAllocateBuffer(sbEntry.cb, (LPVOID*)&(updatedMessages.lpbin[updatedMessages.cValues].lpb) );
                memmove(updatedMessages.lpbin[updatedMessages.cValues].lpb, sbEntry.lpb, sbEntry.cb);
                updatedMessages.cValues++;

                // Free each element in the pRows buffer
                FreeProws(pRows);
            }; break;

            case NO_ACTION_MAIL:
                // No action to do
                break;
        }

    }
    if (mailInfo->getMailItems()->size() > 0)
        *deletedMessages = mailInfo->getMailItems()->clone();

FuncExit:
    if(pRows)
    {
        // Free each element in the pRows buffer
        FreeProws(pRows);
    }
    if (mailInfo) {
        delete mailInfo; mailInfo = NULL;
    }

    RELEASE_OBJ(pContentsTable);
    LOG.debug("Exiting from GetModificatedMessages function");
    return hr;
}

void MailClient2Server::setFolderToSync(const wchar_t* t){
    if(folderToSync)
        delete folderToSync;
    folderToSync = wstrdup(t);
}

wchar_t* MailClient2Server::getFolderToSync() {
    return folderToSync;
}

HRESULT MailClient2Server::setSync (int mode, const wchar_t* path)
{
    LOG.debug("Enter into setSync function");
    HRESULT           hr          =  E_FAIL;
    IMAPISession *    pSession    =  NULL;
    IMAPITable*       pTable      =  NULL;
    SRowSet*          psrs        =  NULL;
    IMsgStore*        pStore      =  NULL;
    int               i           =  0;
    ULONG*   rgTags;
    folderNumber = 0;
    
    if (folderToSync == NULL) {
        rgTags = new ULONG[2];
        rgTags[0] = 1;
        rgTags[1] = PR_CE_IPM_INBOX_ENTRYID;
        folderNumber = 1;
    }
    else {
        folderNumber = wcslen(folderToSync);
        rgTags = new ULONG[folderNumber + 1];
        rgTags[0] = folderNumber;
        for (int i = 1; i <= folderNumber; i++)
            rgTags[i] = getFolderId(folderToSync[i-1]);    // PR_CE_IPM_INBOX_ENTRYID;
        // PR_CE_IPM_DRAFTS_ENTRYID;
    }

    ULONG             rgMsgTags[] =  { 1, PR_ENTRYID };
    ULONG             cValues     =  0;
    ULONG             returned    =  0;

    SPropValue*       rgprops     =  NULL;
    SPropValue*       rgpropsMsg  =  NULL;
    LPMAPIFOLDER      pfldrInbox  =  NULL;

    IMessage*         pmsg        =  NULL;
    ULONG             cmsgValue   =  0;
    SPropValue*       msgprops    =  NULL;


    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    EXIT_ON_FAILED (hr);

    // Get the message stores table
    hr = pSession->GetMsgStoresTable (MAPI_UNICODE, &pTable);
    EXIT_ON_FAILED (hr);

    hr = pTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);

    int totalMails =0;

    while (1){
        // Get a row
        hr = pTable->QueryRows (1, 0, &psrs);
        EXIT_ON_FAILED (hr);

        // Did we hit the end of the table?
        if (psrs->cRows != 1) {
            break;
        }
        //
        // the display name is the name of the account.
        //
        wchar_t* displayName = psrs->aRow[0].lpProps[eePR_DISPLAY_NAME].Value.lpszW;


        //
        // The name of the MessageStore that we use for the own Transport
        //
        if (wcscmp(displayName, ACCOUNT_NAME) == 0) {

            // Make sure we got the props we need
            if ( (psrs->aRow[0].cValues < 1) ||
                 (psrs->aRow[0].lpProps[0].ulPropTag != PR_ENTRYID) ) {
                MessageBox (NULL, L"Store missing PR_ENTRYID!", L"Doh", MB_OK);
                hr = E_FAIL;
                break;
            }
            wchar_t accountMail[256];
            getAccountInfoSession(accName, accountMail, pSession);
            // Open this message store
            hr = pSession->OpenMsgStore (NULL,
                    psrs->aRow[0].lpProps[0].Value.bin.cb,
                    (ENTRYID *) psrs->aRow[0].lpProps[0].Value.
                    bin.lpb, NULL, 0, &pStore);
            EXIT_ON_FAILED (hr);

            // Now get the Inbox folder.
            hr = pStore->GetProps ((SPropTagArray *) rgTags, MAPI_UNICODE, &cValues, &rgprops);
            EXIT_ON_FAILED (hr);

            ASSERT (rgprops);

            entryListArray       = new ENTRYLIST[folderNumber];
            newMessagesArray     = new ENTRYLIST[folderNumber];
            updatedMessagesArray = new ENTRYLIST[folderNumber];
            deletedMessagesArray = new ArrayList*[folderNumber];
            
            //
            // watch at the folder that we want to sync
            //
            for (i = 0; i < folderNumber; i++) {

                // open the message store
                hr = pStore->OpenEntry (rgprops[i].Value.bin.cb,
                        (LPENTRYID)rgprops[i].Value.bin.lpb,
                        NULL,
                        MAPI_MODIFY,
                        NULL,
                        (IUnknown **) &pfldrInbox);
                EXIT_ON_FAILED (hr);

                if ( !pfldrInbox) {
                    LOG.error("Null inbox folder");
                    hr = E_FAIL;
                    break;
                }

                entryListArray[i].cValues = 0;
                newMessagesArray[i].cValues = 0;
                updatedMessagesArray[i].cValues = 0;
                deletedMessagesArray[i] = new ArrayList();

                // ENTRYLIST entry;
                switch (mode) {

                    case ONLY_ALL_MESSAGE_LIST:
                    {
                        hr = GetAllMessages(entryListArray[i], pfldrInbox, path, folderToSync[i]);
                        break;
                    }
                    case SYNC_SLOW:
                    {
                        //extra.clear();

                        if (folderToSync[i] == 'I' || folderToSync[i] == 'S') {
                            ENTRYLIST entry;
                            entry.cValues = 0;
                            hr = GetAllMessages(entry, pfldrInbox, path, folderToSync[i]);
                            hr = pfldrInbox->DeleteMessages (&entry, 0, NULL, 0);

                        } else {

                            hr = GetAllMessages(entryListArray[i], pfldrInbox, path, folderToSync[i]);
                            MailInfo* mailInfo = NULL;
                            readPreviousMail(path, folderToSync[i]);
                            if (mailInfo) {
                                delete mailInfo; mailInfo = NULL;
                            }
                        }
                        /*
                        * for the slow sync of Inbox and Sent (if checked to sync) the new way to do is delete all the items
                        * in the folder.

                        if (folderToSync[i] == 'I' || folderToSync[i] == 'S') {
                            hr = pfldrInbox->DeleteMessages (&entryListArray[i], 0, NULL, 0);
                            //MAPIFreeBuffer(entryListArray[i].lpbin);
                            //entryListArray[i].cValues = 0;
                        } else {
                            readPreviousMail(path, folderToSync[i], extra);
                            if (mailInfo) {
                                delete mailInfo; mailInfo = NULL;
                            }
                        }*/
                        break;
                    }
                    case SYNC_TWO_WAY:
                    {
                        
                        hr = GetModificatedMessages(newMessagesArray[i], updatedMessagesArray[i],
                                &deletedMessagesArray[i], pfldrInbox, path, folderToSync[i]);

                        totalMails += newMessagesArray[i].cValues;
                        totalMails += updatedMessagesArray[i].cValues;
                        totalMails += deletedMessagesArray[i]->size();

                        break;
                    }
                    default:
                        break;
                }
            }
        }


        // Clean up
        MAPIFreeBuffer (rgprops);
        MAPIFreeBuffer (rgpropsMsg);
        FreeProws   (psrs);

        rgprops   =  NULL;
        rgpropsMsg =  NULL;
        psrs    =  NULL;

        RELEASE_OBJ (pmsg);
        RELEASE_OBJ (pfldrInbox);
        RELEASE_OBJ (pStore);
    }

FuncExit:

    MAPIFreeBuffer (rgprops);
    MAPIFreeBuffer (rgpropsMsg);
    FreeProws   (psrs);

    RELEASE_OBJ (pmsg);
    RELEASE_OBJ (pfldrInbox);
    RELEASE_OBJ (pStore);
    RELEASE_OBJ (pTable);
    RELEASE_OBJ (pSession);

    if (rgTags)
        delete rgTags;

    if(HwndFunctions::wnd)
        SendMessage(HwndFunctions::wnd, ID_MYMSG_TOTAL_ITEMS, SOURCE_MAIL, (LPARAM)totalMails);

    LOG.debug("Exiting from setSync function");
    return hr;
}

/*
* Write all the current mail in the message store to get the id, the last, the read
*/

void MailClient2Server::writeCurrentMails(const wchar_t* path) {

    SYSTEMTIME local;
    DATE last;
    IMAPISession *    pSession    = NULL;
    HRESULT           hr          = NULL;
    IMessage*         pmsg        = NULL;
    ULONG             returned    = 0;
    ULONG             cmsgValue   = 0;
    SPropValue*       msgprops    = NULL;
    int j = 0;
    char filename [128];
    char* id = NULL;

    BOOL read = FALSE;

    wchar_t lastTime[64];
    GetSystemTime (&local);
    SystemTimeToVariantTime (&local, &last);
    doubleToSystemTime(lastTime, last);
    //wsprintf(lastTime, TEXT("%i-%i-%iT%i-%i-%i"), local.wYear, local.wMonth, local.wDay, local.wHour, local.wMinute, local.wSecond);

    for (int i = 0; i < folderNumber; i++) {

        //this permits to do not write the mails_O file cache
        if (folderToSync[i] == 'O')
            continue;

        int mailNum = entryListArray[i].cValues;

        FILE* f = NULL;
        if (wcscmp(path, TEXT("\\")) == 0)
            sprintf(filename, "%ls%s_%c",path, "mails", folderToSync[i]);
        else
            sprintf(filename, "%ls\\%s_%c",path, "mails", folderToSync[i]);

        f = fopen(filename, "w+");
        fprintf(f, "<mails>\n");

        fwprintf(f, TEXT("<num>%i</num>\n"), mailNum);
        fwprintf(f, TEXT("<last>%f</last>\n"), last);
        fwprintf(f, TEXT("<lastTime>%s</lastTime>\n"), lastTime);

        pSession = logOn();

        if (pSession == NULL)
            goto FuncExit;

        for (j=0; j<mailNum; j++) {

            hr = pSession->OpenEntry(entryListArray[i].lpbin[j].cb,
                (ENTRYID*)entryListArray[i].lpbin[j].lpb,
                NULL,
                MAPI_BEST_ACCESS,
                &returned,
                (IUnknown **) &pmsg);
            EXIT_ON_FAILED (hr);
            ASSERT (pmsg);

            hr = pmsg->GetProps ((LPSPropTagArray)&endSync, MAPI_UNICODE, &cmsgValue,
                &msgprops);
            fwprintf(f, TEXT("<item>\n"));

            if (msgprops[eeePR_ENTRYID].ulPropTag == PR_ENTRYID) {
                id = convertBinaryToChar(msgprops[eeePR_ENTRYID].Value.bin, folderToSync[i]);
            }
            fprintf(f, "<id>%s</id>\n", id);

            if (msgprops[eeePR_MESSAGE_FLAGS].ulPropTag == PR_MESSAGE_FLAGS) {
                if (msgprops[eeePR_MESSAGE_FLAGS].Value.ul & MSGFLAG_READ) {
                    read = TRUE;
                }
                else {
                    read = FALSE;
                }
            }
            fwprintf(f, TEXT("<read>%i</read>\n"), read);

            fwprintf(f, TEXT("</item>\n"));

            if (id) {
                delete [] id; id = NULL;
            }
        }
        fwprintf(f, TEXT("</mails>\n"));
        fflush(f);
        fclose(f);
    }

FuncExit:

    logOff(pSession);
}

/**
* <mails>
*  <num>2</num>
*  <last>2334.4656</last>
*  <item>
*   <id>AAAAABBRD==</id>
*   <read>0</read>    // if the message is read (1) or not (0)
*  </item>
*  <item>
*   <id>AAAAABBRDFFJJJF==</id>
*   <read>1</read>
*  </item>
* </mails>
*/

MailInfo* MailClient2Server::readPreviousMail(const wchar_t* path,
                                              wchar_t folderId) {

    char filename [128];
    wchar_t* element = NULL;
    wchar_t line[512];
    FILE* f = NULL;
    MailItem mailItem;
    StringBuffer s = "";
    int num = -1;

    if (wcscmp(path, TEXT("\\")) == 0)
        sprintf(filename, "%ls%s_%c",path, "mails", folderId);
    else
        sprintf(filename, "%ls\\%s_%c",path, "mails", folderId);

    f = fopen(filename, "r");

    // no file so a slow sync is perfomed
    if (f == NULL)
        return NULL;

    MailInfo* mailInfo = new MailInfo();
    
    while(fgetws(line, 511, f) != NULL) {

        if ((element = getElementContent(line, TEXT("num"), NULL)) != NULL) {
            mailInfo->setNum(element);
        }
        else if ((element = getElementContent(line, TEXT("last"), NULL)) != NULL) {
            mailInfo->setLast(element);
        }
        else if (wcsstr(line, TEXT("<item>")) != NULL) {
            mailItem = MailItem();
        }
        else if ((element = getElementContent(line, TEXT("id"), NULL)) != NULL) {
            char *e = toMultibyte(element);
            mailItem.setId(e);            
            delete [] e;
        }
        else if ((element = getElementContent(line, TEXT("read"), NULL)) != NULL) {
            mailItem.setRead(_wcc(element));
        }        
        else if (wcsstr(line, TEXT("</item>")) != NULL) {
            mailInfo->addMailItem(mailItem);

        }
        if (element) {
            delete [] element; element = NULL;
        }
    }

    fclose(f);

    return mailInfo;
}

HRESULT MailClient2Server::moveMessageFromOutboxToSentItemsFolder(const wchar_t* key) {
    LOG.debug("Enter into moveMessageFromOutboxToSentItemsFolder function");

    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    IMAPITable*       pTable      =  NULL;
    SRowSet*          psrs        =  NULL;
    IMsgStore*        pStore      =  NULL;
    int               i           =  0;
    ULONG*   rgTags;
    folderNumber = 0;
    BOOL accountFound             = FALSE;

    rgTags = new ULONG[3];
    rgTags[0] = 2;
    rgTags[1] = PR_IPM_OUTBOX_ENTRYID;
    rgTags[2] = PR_IPM_SENTMAIL_ENTRYID;

    ULONG             rgMsgTags[] =  { 1, PR_ENTRYID };
    ULONG             cValues     =  0;
    ULONG             returned    =  0;

    SPropValue*       rgprops     =  NULL;
    SPropValue*       rgpropsMsg  =  NULL;
    LPMAPIFOLDER      pfldrSent   =  NULL;
    LPMAPIFOLDER      pfldrOutbox =  NULL;

    IMessage*         pmsg        =  NULL;
    ULONG             cmsgValue   =  0;
    SPropValue*       msgprops    =  NULL;
    IMessage*         pmsgSent    =  NULL;

    wchar_t*          ret          = NULL;
    wchar_t*          displayName  = NULL;
    bool msgset = false;

    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    EXIT_ON_FAILED (hr);

    // Get the message stores table
    hr = pSession->GetMsgStoresTable (MAPI_UNICODE, &pTable);
    EXIT_ON_FAILED (hr);

    hr = pTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);

    while (!msgset){
        // Get a row
        hr = pTable->QueryRows (1, 0, &psrs);
        EXIT_ON_FAILED (hr);

        // Did we hit the end of the table?
        if (psrs->cRows != 1) {
            break;
        }
        //
        // the display name is the name of the account.
        //
        displayName = psrs->aRow[0].lpProps[eePR_DISPLAY_NAME].Value.lpszW;

        //
        // The name of the MessageStore that we use for the own Transport
        //
        if (wcscmp(displayName, ACCOUNT_NAME) == 0) {
            accountFound = TRUE;
            // Make sure we got the props we need
            if ((psrs->aRow[0].cValues < 1) || (psrs->aRow[0].lpProps[0].ulPropTag != PR_ENTRYID))
            {
                MessageBox (NULL, L"Store missing PR_ENTRYID!", L"Doh", MB_OK);
                hr = E_FAIL;
                break;
            }

            // Open this message store
            hr = pSession->OpenMsgStore (NULL,
                           psrs->aRow[0].lpProps[0].Value.bin.cb,
                          (ENTRYID *) psrs->aRow[0].lpProps[0].Value.
                           bin.lpb, NULL, 0, &pStore);
            EXIT_ON_FAILED (hr);

            // Now get the Inbox folder.
            hr = pStore->GetProps ((SPropTagArray *) rgTags, MAPI_UNICODE, &cValues, &rgprops);
            EXIT_ON_FAILED (hr);

            ASSERT (rgprops);

            //
            // open the folder Outbox and Sent
            //

             // open the message store
            hr = pStore->OpenEntry (rgprops[0].Value.bin.cb,
                        (LPENTRYID)rgprops[0].Value.bin.lpb,
                        NULL,
                        MAPI_MODIFY,
                        NULL,
                        (IUnknown **) &pfldrOutbox);
            EXIT_ON_FAILED (hr);
            ASSERT (pfldrOutbox);

            hr = pStore->OpenEntry (rgprops[1].Value.bin.cb,
                        (LPENTRYID)rgprops[1].Value.bin.lpb,
                        NULL,
                        MAPI_MODIFY,
                        NULL,
                        (IUnknown **) &pfldrSent);
            EXIT_ON_FAILED (hr);
            ASSERT (pfldrSent);

             // Decode the id
            char* eid = NULL;
            size_t len = decodeEntryId(key, &eid);
            if(!eid){
                LOG.error("updateMessage: error decoding entryId");
                return -1;
            }

            ENTRYLIST entryList;

            entryList.cValues = 0;
            MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(entryList.lpbin));
            entryList.lpbin[entryList.cValues].cb = len;
            MAPIAllocateBuffer(len, (LPVOID*)&(entryList.lpbin[entryList.cValues].lpb) );
            memmove(entryList.lpbin[entryList.cValues].lpb, eid, len);
            entryList.cValues++;

            hr = pSession->OpenEntry(entryList.lpbin[0].cb,
                (ENTRYID*)entryList.lpbin[0].lpb,
                NULL,
                MAPI_BEST_ACCESS,
                &returned,
                (IUnknown **) &pmsg);

            EXIT_ON_FAILED (hr);
            if(!pmsg){
                LOG.error("Pmsg is null. ");
                goto FuncExit;
            }

            //
            // Add the read flag to the existing
            //
            SPropValue rgprops[3] = {0};
            ULONG    cProps   = 0;
            ULONG    cmsgValue =  0;
            SPropValue* msgprops = NULL;
            ULONG messageFlag = (ULONG) 0x00000000;
            FILETIME ft;
            char* msgId = NULL;

            hr = pmsg->GetProps ((LPSPropTagArray)&properties, MAPI_UNICODE, &cmsgValue,
               &msgprops);
            EXIT_ON_FAILED (hr);

            if (msgprops[ePR_MESSAGE_FLAGS].ulPropTag == PR_MESSAGE_FLAGS) {
                messageFlag = msgprops[ePR_MESSAGE_FLAGS].Value.ul;
            }

            if (msgprops[ePR_LAST_MODIFICATION_TIME].ulPropTag == PR_LAST_MODIFICATION_TIME) {
                ft = msgprops[ePR_LAST_MODIFICATION_TIME].Value.ft;
            }

            // Set the flags
            rgprops[cProps].ulPropTag = PR_MESSAGE_FLAGS;
            rgprops[cProps].Value.ul = messageFlag | MSGFLAG_READ | MSGFLAG_FROMME;
            ++cProps;

            rgprops[cProps].ulPropTag = PR_CLIENT_SUBMIT_TIME;
            rgprops[cProps].Value.ft = ft;
            ++cProps;

            rgprops[cProps].ulPropTag = PR_MESSAGE_DELIVERY_TIME;
            rgprops[cProps].Value.ft = ft;
            ++cProps;

            hr = pmsg->SetProps(cProps, rgprops, NULL);
            EXIT_ON_FAILED(hr);

            HRESULT hr = pfldrOutbox->CopyMessages(
                            &entryList,
                            NULL,
                            pfldrSent,
                            0,
                            NULL,
                            MESSAGE_MOVE);

            delete [] eid;

            MAPIFreeBuffer (entryList.lpbin);


            msgset = true;
            break;

        }

        // Clean up
        MAPIFreeBuffer (rgprops);

        MAPIFreeBuffer (rgpropsMsg);
        FreeProws   (psrs);

        rgprops   =  NULL;
        rgpropsMsg =  NULL;
        psrs    =  NULL;

        RELEASE_OBJ (pmsg);
        RELEASE_OBJ (pfldrOutbox);
        RELEASE_OBJ (pfldrSent);
        RELEASE_OBJ (pStore);

    }
    if (accountFound == FALSE) {
        LOG.error("No Funambol account found");
    }
FuncExit:

    MAPIFreeBuffer (rgprops);
    MAPIFreeBuffer (rgpropsMsg);
    FreeProws   (psrs);

    RELEASE_OBJ (pmsg);
    RELEASE_OBJ (pfldrOutbox);
    RELEASE_OBJ (pfldrSent);
    RELEASE_OBJ (pStore);
    RELEASE_OBJ (pTable);
    RELEASE_OBJ (pSession);

    if (rgTags)
        delete rgTags;
    LOG.debug("Exiting from moveMessageFromOutboxToSentItemsFolder function");

    return hr;
}
