/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */



#ifndef INCL_WINSYNC_SOURCE_CONFIG
#define INCL_WINSYNC_SOURCE_CONFIG

#include "spds/SyncSourceConfig.h"

/*
* --------------------- WindowsSyncSourceConfig class -----------------------------------
* This class groups all configuration properties for a WindowsSyncSource.
* WindowsSyncSourceConfig has an external link to SyncSourceConfig ('s'), where all common
* properties are stored. Client-specific properties of WindowsSyncSource are members
* of this class.
*
*
* (see spds/SyncSourceConfig.h for common members)
*
*/

#include <string>
using namespace std;

class WindowsSyncSourceConfig {

private:
    // for the sources
    string useSif; //contact, calendar, task
    string dir; //briefcase, notes
    string schedule;

    // Pointer to (external) original SyncSourceConfig object, to retrieve
    // all common properties: we MUST get/set common properties from
    // a unique place, so get/set methods for common properties are overrided
    // and linked to  methods of original SyncSourceConfig 's'.
    SyncSourceConfig* s;


    // Initialize all members.
    void initialize();


public:

    // Default constructor - please note that 's' pointer MUST be set!!
    WindowsSyncSourceConfig();
    //Constructs a new WindowsSyncSourceConfig object.
    // Initialize members and link the SyncSourceConfig pointer passed (mandatory not NULL).
    WindowsSyncSourceConfig(SyncSourceConfig* sc);

    // Copy constructor
    WindowsSyncSourceConfig(const WindowsSyncSourceConfig& wsc);

    //Destructor
    ~WindowsSyncSourceConfig();

    // Operator =
    WindowsSyncSourceConfig& operator = (const WindowsSyncSourceConfig& wsc);

    // Assign the internal SyncSourceConfig* pointer.
    void setCommonConfig(SyncSourceConfig* sc);

    // Return the pointer to external SyncSourceConfig object
    // used for common properties.
    SyncSourceConfig* getCommonConfig();

    // Common properties: get original values in SSconfig 's'
    const char*   getName()           const     { return s->getName()          ; }
    const char*   getURI()            const     { return s->getURI()           ; }
    const char*   getSyncModes()      const     { return s->getSyncModes()     ; }
    const char*   getType()           const     { return s->getType()          ; }
    const char*   getSync()           const     { return s->getSync()          ; }
    const char*   getEncoding()       const     { return s->getEncoding()      ; }
    const char*   getVersion()        const     { return s->getVersion()       ; }
    const char*   getSupportedTypes() const     { return s->getSupportedTypes(); }
    CTCap         getCtCap()          const     { return s->getCtCap()         ; }
    unsigned long getLast()           const     { return s->getLast()          ; }
    const char*   getEncryption()     const     { return s->getEncryption()    ; }
    const string& getUseSif() const;//         const     { return useSif;                 }

    __declspec(dllexport) const string& getSyncDir() const;
    const string& getSchedule()       const     { return schedule;               }

    // Common properties: set original values in SSconfig 's'
    void setName          (const char*   v)     { s->setName(v)                ; }
    void setURI           (const char*   v)     { s->setURI(v)                 ; }
    void setSyncModes     (const char*   v)     { s->setSyncModes(v)           ; }
    void setType          (const char*   v)     { s->setType(v)                ; }
    void setSync          (const char*   v)     { s->setSync(v)                ; }
    void setEncoding      (const char*   v)     { s->setEncoding(v)            ; }
    void setVersion       (const char*   v)     { s->setVersion(v)             ; }
    void setSupportedTypes(const char*   v)     { s->setSupportedTypes(v)      ; }
    void setCtCap         (CTCap         v)     { s->setCtCap(v)               ; }
    void setLast          (unsigned long v)     { s->setLast(v)                ; }
    void setEncryption    (const char*   v)     { s->setEncryption(v)          ; }
    void setUseSif        (const string& v)     { useSif = v; }
    void setSyncDir           (const string& v)     { dir = v; }
    void setSchedule      (const string& v)     { schedule = v; }
};

#endif
