/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */

#ifndef __OTA_CONFIG_MESSAGE_H__
#define __OTA_CONFIG_MESSAGE_H__

#include <windows.h>
#include "pushclient.h"
#include "spds/constants.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreefactory.h"
#include "base/Log.h"
#include "pim/utils.h"
#include "base/util/utils.h"

class OTAConfigMessage  {

private:

    const char* syncURL;
    const char* userName;
    const char* password;

    const char* visibleName;
    const char* mailAddress;

    // set the uri of the source. If NULL never set
    const char* mailURI;
    const char* contactURI;
    const char* calendarURI;
    const char* taskURI;
    const char* noteURI;
    const char* briefcaseURI;

    // set the format of the source. If NULL never set
    const char* contactFormat;
    const char* calendarFormat;
    const char* taskFormat;

    // set the localURI
    const char* noteLocalURI;
    const char* briefcaseLocalURI;


public:

    /*
    * return an ota config message
    */
    OTAConfigMessage();
    ~OTAConfigMessage();

    void setUsername(const char* user);
    const char* getUsername() { return userName; }

    void setPassword(const char* pass);
    const char* getPassword() { return password; }

    void setSyncURL(const char* url);
    const char* getSyncURL() { return syncURL; }

    void setVisibleName(const char* name);
    const char* getVisibleName() { return visibleName; }

    void setMailAddress(const char* address);
    const char* getMailAddress() { return mailAddress; }

    void setMailURI(const char* uri);
    const char* getMailURI() { return mailURI; }

    void setContactURI(const char* uri);
    const char* getContactURI() { return contactURI; }

    void setCalendarURI(const char* uri);
    const char* getCalendarURI() { return calendarURI; }

    void setTaskURI(const char* uri);
    const char* getTaskURI() { return taskURI; }

    void setNoteURI(const char* uri);
    const char* getNoteURI() { return noteURI; }

    void setBriefcaseURI(const char* uri);
    const char* getBriefcaseURI() { return briefcaseURI; }

    void setContactFormat(const char* uri);
    const char* getContactFormat() { return contactFormat; }

    void setCalendarFormat(const char* uri);
    const char* getCalendarFormat() { return calendarFormat; }

    void setTaskFormat(const char* uri);
    const char* getTaskFormat() { return taskFormat; }

    void setNoteLocalURI(const char* uri);
    const char* getNoteLocalURI() { return noteLocalURI; }

    void setBriefcaseLocalURI(const char* uri);
    const char* getBriefcaseLocalURI() { return briefcaseLocalURI; }


    /*
    * return true if the action goes properly; false othewhise
    */
    static bool store(OTAConfigMessage otaMsg);

};


#endif

