/*
 * Copyright (C) 2003-2007 Funambol, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 */


#include "pushmessage.h"
#include "logutils.h"

int handleWapPushMessage(LPPUSHMSG pPushMsg, wchar_t*** retsources)
{
    int ret             = 0;
    wchar_t* srcname    = NULL;
    SyncNotification    np;
    SN_Errors           errcode;
    SyncAlert*          sync;

    wchar_t **sources;

    if (pPushMsg->pbBody == NULL)  {
        LOG.error("Wap push: sms body is empty!");
        ret =  -1;
        goto finally;
    }

    // only to debug
    SaveFile(pPushMsg);

    errcode = np.parse((const char*)pPushMsg->pbBody, pPushMsg->cbBodyLength);

    PrintMessage(np);

    int n = np.getNumSyncs();
    int count = 0;

    if (n > 0) {
        sources = new wchar_t*[n+1];

        for (int i = 0; i < n; i++) {
            sync = np.getSyncAlert(i);

            if( !sync ){
                LOG.error("Wap push: no SyncAlert in SyncNotification");
                ret = -1;
                goto finally;
            }
            srcname = getSourceName(APPLICATION_URI, sync->getServerURI());
            if (srcname) {
                sources[count++] = srcname;
            } else {
                LOG.error("SMS notification: no source found from server notification request: %s", sync->getServerURI());
            }
        }
        sources[count] = NULL;
        *retsources = sources;
    }
    else {
        LOG.info("SMS notification: no sources to sync from server");
        *retsources = NULL;
    }

finally:
    return ret;
}



static wchar_t *getSourceName(const char *root, const char *uri)
{
    int namelen = strlen(root) + strlen(CONTEXT_SPDS_SOURCES) + 5;
    int i = 0;
    char *nodeName = new char[namelen];
    wchar_t *ret = NULL;

    if(!nodeName)
        return NULL;

    sprintf(nodeName, "%s/%s", root, CONTEXT_SPDS_SOURCES);

	DMTree* dmt = DMTreeFactory::getDMTree(root);

    ManagementNode *node = dmt->readManagementNode(nodeName);
    if ( ! node ) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        sprintf(lastErrorMsg, ERRMSG_INVALID_CONTEXT, nodeName);
        DebugPrint(TEXT("getSourceName: error: %s"), lastErrorMsg);
        goto finally;
    }
    for(i=0; i<node->getChildrenMaxCount(); i++) {
        ManagementNode *src = node->getChild(i);
        char *tmp = src->readPropertyValue(PROPERTY_SOURCE_URI);
        //DebugPrint(TEXT("i:%d uri:%s tmp:%s\n"), i, uri, tmp);
        if (strcmp(uri, tmp) == 0) {
            char *val = src->readPropertyValue(PROPERTY_SOURCE_NAME);
            ret = toWideChar(val);
            delete [] val;
            break;
        }
        delete [] tmp;
    }

finally:
    if (dmt) {
        delete dmt;
    }
    if (node) {
        delete node;
    }
    return ret;
}
